@@ -1,7 +1,7 @@
 /* aztec.c - Handles Aztec 2D Symbols */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2009-2023 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -37,10 +37,15 @@
 #include "reedsol.h"
 
 #define AZTEC_MAX_CAPACITY  19968 /* ISO/IEC 24778:2008 5.3 Table 1 Maximum Symbol Bit Capacity */
-#define AZTEC_BIN_CAPACITY  17940 /* Above less 169 * 12 = 2028 bits (169 = 10% of 1664 + 3) */
+/* Allow up to absolute minimum 3 ECC codewords, but now warn if results in less than the 5% minimum (ISO/IEC
+   24778:2008 4.1.e) - previously could go down to 3 ECC codewords anyway if version given, due to bit-stuffing */
+#define AZTEC_BIN_CAPACITY  19932 /* AZTEC_MAX_CAPACITY less 3 * 12 = 36 */
 #define AZTEC_MAP_SIZE      22801 /* AztecMap Version 32 151 x 151 */
 #define AZTEC_MAP_POSN_MAX  20039 /* Maximum position index in AztecMap */
 
+#define AZ_BIN_CAP_CWDS_S   "1661" /* String version of (AZTEC_BIN_CAPACITY / 12) */
+
+/* Count number of consecutive (. SP) or (, SP) Punct mode doubles for comparison against Digit mode encoding */
 static int az_count_doubles(const unsigned char source[], int i, const int length) {
     int c = 0;
 
@@ -52,6 +57,7 @@ static int az_count_doubles(const unsigned char source[], int i, const int lengt
     return c;
 }
 
+/* Count number of consecutive full stops or commas (can be encoded in Punct or Digit mode) */
 static int az_count_dotcomma(const unsigned char source[], int i, const int length) {
     int c = 0;
 
@@ -63,6 +69,7 @@ static int az_count_dotcomma(const unsigned char source[], int i, const int leng
     return c;
 }
 
+/* Count number of consecutive `chr`s */
 static int az_count_chr(const unsigned char source[], int i, const int length, const unsigned char chr) {
     int c = 0;
 
@@ -74,6 +81,7 @@ static int az_count_chr(const unsigned char source[], int i, const int length, c
     return c;
 }
 
+/* Return mode following current, or 'E' if none */
 static char az_get_next_mode(const char encode_mode[], const int src_len, int i) {
     int current_mode = encode_mode[i];
 
@@ -87,6 +95,7 @@ static char az_get_next_mode(const char encode_mode[], const int src_len, int i)
     }
 }
 
+/* Same as `bin_append_posn()`, except check for buffer overflow first */
 static int az_bin_append_posn(const int arg, const int length, char *binary, const int bin_posn) {
 
     if (bin_posn + length > AZTEC_BIN_CAPACITY) {
@@ -95,6 +104,7 @@ static int az_bin_append_posn(const int arg, const int length, char *binary, con
     return bin_append_posn(arg, length, binary, bin_posn);
 }
 
+/* Determine encoding modes and encode */
 static int aztec_text_process(const unsigned char source[], int src_len, int bp, char binary_string[], const int gs1,
             const int eci, char *p_current_mode, int *data_length, const int debug_print) {
 
@@ -111,8 +121,10 @@ static int aztec_text_process(const unsigned char source[], int src_len, int bp,
     for (i = 0; i < src_len; i++) {
         if (source[i] >= 128) {
             encode_mode[i] = 'B';
+        } else if (gs1 && source[i] == '\x1D') {
+            encode_mode[i] = 'P'; /* For FLG(n) & FLG(0) = FNC1 */
         } else {
-            encode_mode[i] = AztecModes[(int) source[i]];
+            encode_mode[i] = AztecModes[source[i]];
         }
     }
 
@@ -179,32 +191,25 @@ static int aztec_text_process(const unsigned char source[], int src_len, int bp,
     i = 0;
     j = 0;
     while (i < src_len) {
+        reduced_encode_mode[j] = encode_mode[i];
         if (i + 1 < src_len) {
             if ((source[i] == 13) && (source[i + 1] == 10)) { /* CR LF */
                 reduced_source[j] = 'a';
-                reduced_encode_mode[j] = encode_mode[i];
                 i += 2;
             } else if ((source[i] == '.') && (source[i + 1] == ' ') && (encode_mode[i] == 'P')) {
                 reduced_source[j] = 'b';
-                reduced_encode_mode[j] = encode_mode[i];
                 i += 2;
             } else if ((source[i] == ',') && (source[i + 1] == ' ') && (encode_mode[i] == 'P')) {
                 reduced_source[j] = 'c';
-                reduced_encode_mode[j] = encode_mode[i];
                 i += 2;
             } else if ((source[i] == ':') && (source[i + 1] == ' ')) {
                 reduced_source[j] = 'd';
-                reduced_encode_mode[j] = encode_mode[i];
                 i += 2;
             } else {
-                reduced_source[j] = source[i];
-                reduced_encode_mode[j] = encode_mode[i];
-                i++;
+                reduced_source[j] = source[i++];
             }
         } else {
-            reduced_source[j] = source[i];
-            reduced_encode_mode[j] = encode_mode[i];
-            i++;
+            reduced_source[j] = source[i++];
         }
         j++;
     }
@@ -576,7 +581,9 @@ static int aztec_text_process(const unsigned char source[], int src_len, int bp,
                 int big_batch = 0;
                 for (count = 0; ((i + count) < reduced_length) && (reduced_encode_mode[i + count] == 'B'); count++);
 
-                assert(count <= 2047 + 2078); /* Can't be more than 19968 / 8 = 2496 */
+                if (count > 2047 + 2078) { /* Can't be more than 19968 / 8 = 2496 */
+                    return 0;
+                }
 
                 if (count > 2047) { /* Max 11-bit number */
                     big_batch = count > 2078 ? 2078 : count;
@@ -616,15 +623,13 @@ static int aztec_text_process(const unsigned char source[], int src_len, int bp,
             if (reduced_source[i] == ' ') {
                 if (!(bp = az_bin_append_posn(1, 5, binary_string, bp))) return 0; /* SP */
             } else {
-                if (!(bp = az_bin_append_posn(AztecSymbolChar[(int) reduced_source[i]], 5, binary_string, bp)))
-                    return 0;
+                if (!(bp = az_bin_append_posn(AztecSymbolChar[reduced_source[i]], 5, binary_string, bp))) return 0;
             }
         } else if (reduced_encode_mode[i] == 'L') {
             if (reduced_source[i] == ' ') {
                 if (!(bp = az_bin_append_posn(1, 5, binary_string, bp))) return 0; /* SP */
             } else {
-                if (!(bp = az_bin_append_posn(AztecSymbolChar[(int) reduced_source[i]], 5, binary_string, bp)))
-                    return 0;
+                if (!(bp = az_bin_append_posn(AztecSymbolChar[reduced_source[i]], 5, binary_string, bp))) return 0;
             }
         } else if (reduced_encode_mode[i] == 'M') {
             if (reduced_source[i] == ' ') {
@@ -632,11 +637,10 @@ static int aztec_text_process(const unsigned char source[], int src_len, int bp,
             } else if (reduced_source[i] == 13) {
                 if (!(bp = az_bin_append_posn(14, 5, binary_string, bp))) return 0; /* CR */
             } else {
-                if (!(bp = az_bin_append_posn(AztecSymbolChar[(int) reduced_source[i]], 5, binary_string, bp)))
-                    return 0;
+                if (!(bp = az_bin_append_posn(AztecSymbolChar[reduced_source[i]], 5, binary_string, bp))) return 0;
             }
         } else if ((reduced_encode_mode[i] == 'P') || (reduced_encode_mode[i] == 'p')) {
-            if (gs1 && (reduced_source[i] == '[')) {
+            if (gs1 && reduced_source[i] == '\x1D') {
                 if (!(bp = az_bin_append_posn(0, 5, binary_string, bp))) return 0; /* FLG(n) */
                 if (!(bp = az_bin_append_posn(0, 3, binary_string, bp))) return 0; /* FLG(0) = FNC1 */
             } else if (reduced_source[i] == 13) {
@@ -654,8 +658,7 @@ static int aztec_text_process(const unsigned char source[], int src_len, int bp,
             } else if (reduced_source[i] == '.') {
                 if (!(bp = az_bin_append_posn(19, 5, binary_string, bp))) return 0; /* Full stop */
             } else {
-                if (!(bp = az_bin_append_posn(AztecSymbolChar[(int) reduced_source[i]], 5, binary_string, bp)))
-                    return 0;
+                if (!(bp = az_bin_append_posn(AztecSymbolChar[reduced_source[i]], 5, binary_string, bp))) return 0;
             }
         } else if (reduced_encode_mode[i] == 'D') {
             if (reduced_source[i] == ' ') {
@@ -665,8 +668,7 @@ static int aztec_text_process(const unsigned char source[], int src_len, int bp,
             } else if (reduced_source[i] == '.') {
                 if (!(bp = az_bin_append_posn(13, 4, binary_string, bp))) return 0; /* Full stop */
             } else {
-                if (!(bp = az_bin_append_posn(AztecSymbolChar[(int) reduced_source[i]], 4, binary_string, bp)))
-                    return 0;
+                if (!(bp = az_bin_append_posn(AztecSymbolChar[reduced_source[i]], 4, binary_string, bp))) return 0;
             }
         }
     }
@@ -714,7 +716,7 @@ static int az_avoidReferenceGrid(int output) {
 
 /* Calculate the position of the bits in the grid (non-compact) */
 static void az_populate_map(short AztecMap[], const int layers) {
-    int layer, n, i;
+    int layer;
     int x, y;
     const int offset = AztecOffset[layers - 1];
     const int endoffset = 151 - offset;
@@ -723,53 +725,50 @@ static void az_populate_map(short AztecMap[], const int layers) {
         const int start = (112 * layer) + (16 * layer * layer) + 2;
         const int length = 28 + (layer * 4) + (layer + 1) * 4;
         int av0, av1;
+        int n = start, end;
         /* Top */
-        i = 0;
         x = 64 - (layer * 2);
         y = 63 - (layer * 2);
         av0 = az_avoidReferenceGrid(y) * 151;
         av1 = az_avoidReferenceGrid(y - 1) * 151;
-        for (n = start; n < (start + length); n += 2) {
-            int avxi = az_avoidReferenceGrid(x + i);
-            AztecMap[av0 + avxi] = n;
-            AztecMap[av1 + avxi] = n + 1;
-            i++;
+        end = start + length;
+        while (n < end) {
+            const int avxi = az_avoidReferenceGrid(x++);
+            AztecMap[av0 + avxi] = n++;
+            AztecMap[av1 + avxi] = n++;
         }
         /* Right */
-        i = 0;
         x = 78 + (layer * 2);
         y = 64 - (layer * 2);
         av0 = az_avoidReferenceGrid(x);
         av1 = az_avoidReferenceGrid(x + 1);
-        for (n = start + length; n < (start + (length * 2)); n += 2) {
-            int avyi = az_avoidReferenceGrid(y + i) * 151;
-            AztecMap[avyi + av0] = n;
-            AztecMap[avyi + av1] = n + 1;
-            i++;
+        end += length;
+        while (n < end) {
+            const int avyi = az_avoidReferenceGrid(y++) * 151;
+            AztecMap[avyi + av0] = n++;
+            AztecMap[avyi + av1] = n++;
         }
         /* Bottom */
-        i = 0;
         x = 77 + (layer * 2);
         y = 78 + (layer * 2);
         av0 = az_avoidReferenceGrid(y) * 151;
         av1 = az_avoidReferenceGrid(y + 1) * 151;
-        for (n = start + (length * 2); n < (start + (length * 3)); n += 2) {
-            int avxi = az_avoidReferenceGrid(x - i);
-            AztecMap[av0 + avxi] = n;
-            AztecMap[av1 + avxi] = n + 1;
-            i++;
+        end += length;
+        while (n < end) {
+            const int avxi = az_avoidReferenceGrid(x--);
+            AztecMap[av0 + avxi] = n++;
+            AztecMap[av1 + avxi] = n++;
         }
         /* Left */
-        i = 0;
         x = 63 - (layer * 2);
         y = 77 + (layer * 2);
         av0 = az_avoidReferenceGrid(x);
         av1 = az_avoidReferenceGrid(x - 1);
-        for (n = start + (length * 3); n < (start + (length * 4)); n += 2) {
-            int avyi = az_avoidReferenceGrid(y - i) * 151;
-            AztecMap[avyi + av0] = n;
-            AztecMap[avyi + av1] = n + 1;
-            i++;
+        end += length;
+        while (n < end) {
+            const int avyi = az_avoidReferenceGrid(y--) * 151;
+            AztecMap[avyi + av0] = n++;
+            AztecMap[avyi + av1] = n++;
         }
     }
 
@@ -789,7 +788,7 @@ static void az_populate_map(short AztecMap[], const int layers) {
 
 /* Helper to insert dummy '0' or '1's into runs of same bits. See ISO/IEC 24778:2008 7.3.1.2 */
 static int az_bitrun_stuff(const char *binary_string, const int data_length, const int codeword_size,
-            char adjusted_string[AZTEC_MAX_CAPACITY]) {
+            const int data_maxsize, char adjusted_string[AZTEC_MAX_CAPACITY]) {
     int i, j = 0, count = 0;
 
     for (i = 0; i < data_length; i++) {
@@ -801,7 +800,7 @@ static int az_bitrun_stuff(const char *binary_string, const int data_length, con
 
             if (count == 0 || count == (codeword_size - 1)) {
                 /* Codeword of B-1 '0's or B-1 '1's */
-                if (j >= AZTEC_MAX_CAPACITY) {
+                if (j > data_maxsize) {
                     return 0; /* Fail */
                 }
                 adjusted_string[j++] = count == 0 ? '1' : '0';
@@ -813,7 +812,7 @@ static int az_bitrun_stuff(const char *binary_string, const int data_length, con
         } else if (binary_string[i] == '1') { /* Skip B so only counting B-1 */
             count++;
         }
-        if (j >= AZTEC_MAX_CAPACITY) {
+        if (j > data_maxsize) {
             return 0; /* Fail */
         }
         adjusted_string[j++] = binary_string[i];
@@ -869,23 +868,18 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
     int error_number = 0;
     int compact, data_length, data_maxsize, codeword_size, adjusted_length;
     int remainder, padbits, adjustment_size;
-    int reader = 0;
-    int comp_loop = 4;
     int bp = 0;
     const int gs1 = (symbol->input_mode & 0x07) == GS1_MODE;
-    const int debug_print = (symbol->debug & ZINT_DEBUG_PRINT);
+    const int reader_init = symbol->output_options & READER_INIT;
+    const int compact_loop_start = reader_init ? 1 : 4; /* Compact 2-4 excluded from Reader Initialisation */
+    const int debug_print = symbol->debug & ZINT_DEBUG_PRINT;
     rs_t rs;
     rs_uint_t rs_uint;
     unsigned int *data_part;
     unsigned int *ecc_part;
 
-    if (symbol->output_options & READER_INIT) {
-        reader = 1;
-        comp_loop = 1;
-    }
-    if (gs1 && reader) {
-        strcpy(symbol->errtxt, "501: Cannot encode in GS1 and Reader Initialisation mode at the same time");
-        return ZINT_ERROR_INVALID_OPTION;
+    if (gs1 && reader_init) {
+        return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 501, "Cannot use Reader Initialisation in GS1 mode");
     }
 
     if (symbol->structapp.count) {
@@ -895,19 +889,20 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         int id_len;
 
         if (symbol->structapp.count < 2 || symbol->structapp.count > 26) {
-            strcpy(symbol->errtxt, "701: Structured Append count out of range (2-26)");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 701,
+                            "Structured Append count '%d' out of range (2 to 26)", symbol->structapp.count);
         }
         if (symbol->structapp.index < 1 || symbol->structapp.index > symbol->structapp.count) {
-            sprintf(symbol->errtxt, "702: Structured Append index out of range (1-%d)", symbol->structapp.count);
-            return ZINT_ERROR_INVALID_OPTION;
+            return ZEXT errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 702,
+                                "Structured Append index '%1$d' out of range (1 to count %2$d)",
+                                symbol->structapp.index, symbol->structapp.count);
         }
 
         for (id_len = 0; id_len < 32 && symbol->structapp.id[id_len]; id_len++);
 
         if (id_len && chr_cnt((const unsigned char *) symbol->structapp.id, id_len, ' ')) {
-            strcpy(symbol->errtxt, "703: Structured Append ID cannot contain spaces");
-            return ZINT_ERROR_INVALID_OPTION;
+            /* Note ID can contain any old chars apart from space so don't print in error message */
+            return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 703, "Structured Append ID cannot contain spaces");
         }
 
         bp = bin_append_posn(29, 5, binary_string, bp); /* M/L */
@@ -924,7 +919,7 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         sa_src[sa_len++] = 'A' + symbol->structapp.count - 1;
         if (debug_print) {
             printf("Structured Append Count: %d, Index: %d, ID: %.32s, String: %s\n",
-                    symbol->structapp.count, symbol->structapp.count, symbol->structapp.id, sa_src);
+                    symbol->structapp.count, symbol->structapp.index, symbol->structapp.id, sa_src);
         }
 
         (void) aztec_text_process(sa_src, sa_len, bp, binary_string, 0 /*gs1*/, 0 /*eci*/, NULL /*p_current_mode*/,
@@ -933,32 +928,26 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
     }
 
     if (!aztec_text_process_segs(segs, seg_count, bp, binary_string, gs1, &data_length, debug_print)) {
-        strcpy(symbol->errtxt, "502: Input too long or too many extended ASCII characters");
-        return ZINT_ERROR_TOO_LONG;
+        return errtxt(ZINT_ERROR_TOO_LONG, symbol, 502,
+                        "Input too long, requires too many codewords (maximum " AZ_BIN_CAP_CWDS_S ")");
     }
     assert(data_length > 0); /* Suppress clang-tidy warning: clang-analyzer-core.UndefinedBinaryOperatorResult */
 
-    if (!((symbol->option_1 >= -1) && (symbol->option_1 <= 4))) {
-        strcpy(symbol->errtxt, "503: Invalid error correction level - using default instead");
+    if (symbol->option_1 < -1 || symbol->option_1 > 4) {
+        errtxtf(0, symbol, 503, "Error correction level '%d' out of range (1 to 4)", symbol->option_1);
         if (symbol->warn_level == WARN_FAIL_ALL) {
             return ZINT_ERROR_INVALID_OPTION;
         }
-        error_number = ZINT_WARN_INVALID_OPTION;
+        error_number = errtxt_adj(ZINT_WARN_INVALID_OPTION, symbol, "%1$s%2$s", ", ignoring");
         symbol->option_1 = -1;
     }
 
     data_maxsize = 0; /* Keep compiler happy! */
     adjustment_size = 0;
     if (symbol->option_2 == 0) { /* The size of the symbol can be determined by Zint */
-        static const short *full_sizes[5] = {
-            NULL, Aztec10DataSizes, Aztec23DataSizes, Aztec36DataSizes, Aztec50DataSizes
-        };
-        static const short *comp_sizes[5] = {
-            NULL, AztecCompact10DataSizes, AztecCompact23DataSizes, AztecCompact36DataSizes, AztecCompact50DataSizes
-        };
         int ecc_level = symbol->option_1;
 
-        if ((ecc_level == -1) || (ecc_level == 0)) {
+        if (ecc_level <= 0) {
             ecc_level = 2;
         }
 
@@ -968,34 +957,43 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
             layers = 0;
 
             /* For each level of error correction work out the smallest symbol which the data will fit in */
-            for (i = comp_loop; i > 0; i--) {
-                if ((data_length + adjustment_size) < comp_sizes[ecc_level][i - 1]) {
+            for (i = compact_loop_start; i > 0; i--) {
+                if ((data_length + adjustment_size) <= AztecCompactDataSizes[ecc_level - 1][i - 1]) {
                     layers = i;
                     compact = 1;
-                    data_maxsize = comp_sizes[ecc_level][i - 1];
+                    data_maxsize = AztecCompactDataSizes[ecc_level - 1][i - 1];
                 }
             }
             if (!compact) {
                 for (i = 32; i > 0; i--) {
-                    if ((data_length + adjustment_size) < full_sizes[ecc_level][i - 1]) {
+                    if ((data_length + adjustment_size) <= AztecDataSizes[ecc_level - 1][i - 1]) {
                         layers = i;
                         compact = 0;
-                        data_maxsize = full_sizes[ecc_level][i - 1];
+                        data_maxsize = AztecDataSizes[ecc_level - 1][i - 1];
                     }
                 }
             }
 
             if (layers == 0) { /* Couldn't find a symbol which fits the data */
-                strcpy(symbol->errtxt, "504: Input too long (too many bits for selected ECC)");
-                return ZINT_ERROR_TOO_LONG;
+                if (adjustment_size == 0) {
+                    return ZEXT errtxtf(ZINT_ERROR_TOO_LONG, symbol, 707,
+                                    "Input too long for ECC level %1$d, requires too many codewords (maximum %2$d)",
+                                    ecc_level, AztecDataSizes[ecc_level - 1][31] / 12);
+                }
+                return ZEXT errtxtf(ZINT_ERROR_TOO_LONG, symbol, 504,
+                                    "Input too long for ECC level %1$d, requires %2$d codewords (maximum %3$d)",
+                                    ecc_level, (data_length + adjustment_size + 11) / 12,
+                                    AztecDataSizes[ecc_level - 1][31] / 12);
             }
 
             codeword_size = az_codeword_size(layers);
 
-            adjusted_length = az_bitrun_stuff(binary_string, data_length, codeword_size, adjusted_string);
+            adjusted_length = az_bitrun_stuff(binary_string, data_length, codeword_size,
+                                                adjustment_size ? data_maxsize : AZTEC_BIN_CAPACITY, adjusted_string);
             if (adjusted_length == 0) {
-                strcpy(symbol->errtxt, "705: Data too long for specified Aztec Code symbol size");
-                return ZINT_ERROR_TOO_LONG;
+                return ZEXT errtxtf(ZINT_ERROR_TOO_LONG, symbol, 705,
+                                    "Input too long for ECC level %1$d, requires too many codewords (maximum %2$d)",
+                                    ecc_level, (adjustment_size ? data_maxsize : AZTEC_BIN_CAPACITY) / codeword_size);
             }
             adjustment_size = adjusted_length - data_length;
 
@@ -1008,10 +1006,7 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
             }
             if (debug_print) printf("Remainder: %d  Pad bits: %d\n", remainder, padbits);
 
-            if (adjusted_length + padbits >= AZTEC_MAX_CAPACITY) { /* Probably can't happen */
-                strcpy(symbol->errtxt, "706: Data too long for specified Aztec Code symbol size");
-                return ZINT_ERROR_TOO_LONG;
-            }
+            assert(adjusted_length <= AZTEC_BIN_CAPACITY);
 
             adjusted_length = az_add_padding(padbits, codeword_size, adjusted_string, adjusted_length);
 
@@ -1024,11 +1019,19 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
 
     } else { /* The size of the symbol has been specified by the user */
         if ((symbol->option_2 < 0) || (symbol->option_2 > 36)) {
-            strcpy(symbol->errtxt, "510: Invalid Aztec Code size");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 510, "Version '%d' out of range (1 to 36)",
+                            symbol->option_2);
         }
-        if ((reader == 1) && ((symbol->option_2 >= 2) && (symbol->option_2 <= 4))) {
-            symbol->option_2 = 5;
+        if (reader_init) {
+            /* For back-compatibility, silently ignore compact 2-4 requests but error on layers > 22 */
+            if (symbol->option_2 >= 2 && symbol->option_2 <= 4) {
+                symbol->option_2 = 5;
+            } else if (symbol->option_2 > 26) {
+                /* Caught below anyway but catch here also for better feedback */
+                return errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 709,
+                                "Version '%d' out of range for Reader Initialisation symbols (maximum 26)",
+                                symbol->option_2);
+            }
         }
         if (symbol->option_2 <= 4) {
             compact = 1;
@@ -1039,11 +1042,17 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         }
 
         codeword_size = az_codeword_size(layers);
+        if (compact) {
+            data_maxsize = codeword_size * (AztecCompactSizes[layers - 1] - 3);
+        } else {
+            data_maxsize = codeword_size * (AztecSizes[layers - 1] - 3);
+        }
 
-        adjusted_length = az_bitrun_stuff(binary_string, data_length, codeword_size, adjusted_string);
+        adjusted_length = az_bitrun_stuff(binary_string, data_length, codeword_size, data_maxsize, adjusted_string);
         if (adjusted_length == 0) {
-            strcpy(symbol->errtxt, "704: Data too long for specified Aztec Code symbol size");
-            return ZINT_ERROR_TOO_LONG;
+            return ZEXT errtxtf(ZINT_ERROR_TOO_LONG, symbol, 704,
+                                "Input too long for Version %1$d, requires too many codewords (maximum %2$d)",
+                                symbol->option_2, data_maxsize / codeword_size);
         }
 
         /* Add padding */
@@ -1056,15 +1065,12 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         if (debug_print) printf("Remainder: %d  Pad bits: %d\n", remainder, padbits);
 
         /* Check if the data actually fits into the selected symbol size */
-        if (compact) {
-            data_maxsize = codeword_size * (AztecCompactSizes[layers - 1] - 3);
-        } else {
-            data_maxsize = codeword_size * (AztecSizes[layers - 1] - 3);
-        }
 
         if (adjusted_length + padbits > data_maxsize) {
-            strcpy(symbol->errtxt, "505: Data too long for specified Aztec Code symbol size");
-            return ZINT_ERROR_TOO_LONG;
+            return ZEXT errtxtf(ZINT_ERROR_TOO_LONG, symbol, 505,
+                                "Input too long for Version %1$d, requires %2$d codewords (maximum %3$d)",
+                                symbol->option_2, (adjusted_length + padbits) / codeword_size,
+                                data_maxsize / codeword_size);
         }
 
         adjusted_length = az_add_padding(padbits, codeword_size, adjusted_string, adjusted_length);
@@ -1080,9 +1086,9 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         fputc('\n', stdout);
     }
 
-    if (reader && (layers > 22)) {
-        strcpy(symbol->errtxt, "506: Data too long for reader initialisation symbol");
-        return ZINT_ERROR_TOO_LONG;
+    if (reader_init && (layers > 22)) {
+        return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 506,
+                        "Input too long for Reader Initialisation, requires %d layers (maximum 22)", layers);
     }
 
     data_blocks = adjusted_length / codeword_size;
@@ -1095,6 +1101,11 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
     } else {
         ecc_blocks = AztecSizes[layers - 1] - data_blocks;
     }
+    if (ecc_blocks < data_blocks / 20) {
+        error_number = ZEXT errtxtf(ZINT_WARN_NONCOMPLIANT, symbol, 708,
+                                    "Number of ECC codewords %1$d less than %2$d (5%% of data codewords %3$d)",
+                                    ecc_blocks, data_blocks / 20, data_blocks);
+    }
 
     if (debug_print) {
         printf("Generating a %s symbol with %d layers\n", compact ? "compact" : "full-size", layers);
@@ -1131,8 +1142,7 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
             break;
         case 10:
             if (!rs_uint_init_gf(&rs_uint, 0x409, 1023)) { /* Can fail on malloc() */
-                strcpy(symbol->errtxt, "500: Insufficient memory for Reed-Solomon log tables");
-                return ZINT_ERROR_MEMORY;
+                return errtxt(ZINT_ERROR_MEMORY, symbol, 500, "Insufficient memory for Reed-Solomon log tables");
             }
             rs_uint_init_code(&rs_uint, ecc_blocks, 1);
             rs_uint_encode(&rs_uint, data_blocks, data_part, ecc_part);
@@ -1141,8 +1151,7 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         case 12:
             if (!rs_uint_init_gf(&rs_uint, 0x1069, 4095)) { /* Can fail on malloc() */
                 /* Note using AUSPOST error nos range as out of 50x ones & 51x taken by CODEONE */
-                strcpy(symbol->errtxt, "700: Insufficient memory for Reed-Solomon log tables");
-                return ZINT_ERROR_MEMORY;
+                return errtxt(ZINT_ERROR_MEMORY, symbol, 700, "Insufficient memory for Reed-Solomon log tables");
             }
             rs_uint_init_code(&rs_uint, ecc_blocks, 1);
             rs_uint_encode(&rs_uint, data_blocks, data_part, ecc_part);
@@ -1150,7 +1159,7 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
             break;
     }
 
-    for (i = (ecc_blocks - 1); i >= 0; i--) {
+    for (i = 0; i < ecc_blocks; i++) {
         adjusted_length = bin_append_posn(ecc_part[i], codeword_size, adjusted_string, adjusted_length);
     }
 
@@ -1173,7 +1182,7 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         descriptor[1] = ((layers - 1) & 0x01) ? '1' : '0';
 
         /* The next 6 bits represent the number of data blocks minus 1 */
-        descriptor[2] = reader || ((data_blocks - 1) & 0x20) ? '1' : '0';
+        descriptor[2] = reader_init || ((data_blocks - 1) & 0x20) ? '1' : '0';
         for (i = 3; i < 8; i++) {
             descriptor[i] = ((data_blocks - 1) & (0x10 >> (i - 3))) ? '1' : '0';
         }
@@ -1185,7 +1194,7 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         }
 
         /* The next 11 bits represent the number of data blocks minus 1 */
-        descriptor[5] = reader || ((data_blocks - 1) & 0x400) ? '1' : '0';
+        descriptor[5] = reader_init || ((data_blocks - 1) & 0x400) ? '1' : '0';
         for (i = 6; i < 16; i++) {
             descriptor[i] = ((data_blocks - 1) & (0x200 >> (i - 6))) ? '1' : '0';
         }
@@ -1205,19 +1214,19 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         rs_init_code(&rs, 5, 1);
         rs_encode(&rs, 2, desc_data, desc_ecc);
         for (i = 0; i < 5; i++) {
-            descriptor[(i * 4) + 8] = (desc_ecc[4 - i] & 0x08) ? '1' : '0';
-            descriptor[(i * 4) + 9] = (desc_ecc[4 - i] & 0x04) ? '1' : '0';
-            descriptor[(i * 4) + 10] = (desc_ecc[4 - i] & 0x02) ? '1' : '0';
-            descriptor[(i * 4) + 11] = (desc_ecc[4 - i] & 0x01) ? '1' : '0';
+            descriptor[(i * 4) + 8] = (desc_ecc[i] & 0x08) ? '1' : '0';
+            descriptor[(i * 4) + 9] = (desc_ecc[i] & 0x04) ? '1' : '0';
+            descriptor[(i * 4) + 10] = (desc_ecc[i] & 0x02) ? '1' : '0';
+            descriptor[(i * 4) + 11] = (desc_ecc[i] & 0x01) ? '1' : '0';
         }
     } else {
         rs_init_code(&rs, 6, 1);
         rs_encode(&rs, 4, desc_data, desc_ecc);
         for (i = 0; i < 6; i++) {
-            descriptor[(i * 4) + 16] = (desc_ecc[5 - i] & 0x08) ? '1' : '0';
-            descriptor[(i * 4) + 17] = (desc_ecc[5 - i] & 0x04) ? '1' : '0';
-            descriptor[(i * 4) + 18] = (desc_ecc[5 - i] & 0x02) ? '1' : '0';
-            descriptor[(i * 4) + 19] = (desc_ecc[5 - i] & 0x01) ? '1' : '0';
+            descriptor[(i * 4) + 16] = (desc_ecc[i] & 0x08) ? '1' : '0';
+            descriptor[(i * 4) + 17] = (desc_ecc[i] & 0x04) ? '1' : '0';
+            descriptor[(i * 4) + 18] = (desc_ecc[i] & 0x02) ? '1' : '0';
+            descriptor[(i * 4) + 19] = (desc_ecc[i] & 0x01) ? '1' : '0';
         }
     }
 
@@ -1230,15 +1239,13 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
 
     /* Plot all of the data into the symbol in pre-defined spiral pattern */
     if (compact) {
-        int offset = AztecCompactOffset[layers - 1];
-        int end_offset = 27 - offset;
+        const int offset = AztecCompactOffset[layers - 1];
+        const int end_offset = 27 - offset;
         for (y = offset; y < end_offset; y++) {
-            int y_map = y * 27;
+            const int y_map = y * 27;
             for (x = offset; x < end_offset; x++) {
-                int map = AztecCompactMap[y_map + x];
-                if (map == 1) {
-                    set_module(symbol, y - offset, x - offset);
-                } else if (map >= 2 && bit_pattern[map - 2] == '1') {
+                const int map = AztecCompactMap[y_map + x];
+                if (map == 1 || (map >= 2 && bit_pattern[map - 2] == '1')) {
                     set_module(symbol, y - offset, x - offset);
                 }
             }
@@ -1248,16 +1255,14 @@ INTERNAL int aztec(struct zint_symbol *symbol, struct zint_seg segs[], const int
         symbol->rows = 27 - (2 * offset);
         symbol->width = 27 - (2 * offset);
     } else {
-        int offset = AztecOffset[layers - 1];
-        int end_offset = 151 - offset;
+        const int offset = AztecOffset[layers - 1];
+        const int end_offset = 151 - offset;
         az_populate_map(AztecMap, layers);
         for (y = offset; y < end_offset; y++) {
-            int y_map = y * 151;
+            const int y_map = y * 151;
             for (x = offset; x < end_offset; x++) {
-                int map = AztecMap[y_map + x];
-                if (map == 1) {
-                    set_module(symbol, y - offset, x - offset);
-                } else if (map >= 2 && bit_pattern[map - 2] == '1') {
+                const int map = AztecMap[y_map + x];
+                if (map == 1 || (map >= 2 && bit_pattern[map - 2] == '1')) {
                     set_module(symbol, y - offset, x - offset);
                 }
             }
@@ -1283,12 +1288,11 @@ INTERNAL int azrune(struct zint_symbol *symbol, unsigned char source[], int leng
 
     input_value = 0;
     if (length > 3) {
-        strcpy(symbol->errtxt, "507: Input too large (3 character maximum)");
-        return ZINT_ERROR_TOO_LONG;
+        return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 507, "Input length %d too long (maximum 3)", length);
     }
-    if (!is_sane(NEON_F, source, length)) {
-        strcpy(symbol->errtxt, "508: Invalid character in data (digits only)");
-        return ZINT_ERROR_INVALID_DATA;
+    if ((i = not_sane(NEON_F, source, length))) {
+        return errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 508,
+                        "Invalid character at position %d in input (digits only)", i);
     }
     switch (length) {
         case 3:
@@ -1303,8 +1307,7 @@ INTERNAL int azrune(struct zint_symbol *symbol, unsigned char source[], int leng
     }
 
     if (input_value > 255) {
-        strcpy(symbol->errtxt, "509: Input out of range (0 to 255)");
-        return ZINT_ERROR_INVALID_DATA;
+        return errtxt(ZINT_ERROR_INVALID_DATA, symbol, 509, "Input value out of range (0 to 255)");
     }
 
     bp = bin_append_posn(input_value, 8, binary_string, bp);
@@ -1317,7 +1320,7 @@ INTERNAL int azrune(struct zint_symbol *symbol, unsigned char source[], int leng
     rs_encode(&rs, 2, data_codewords, ecc_codewords);
 
     for (i = 0; i < 5; i++) {
-        bp = bin_append_posn(ecc_codewords[4 - i], 4, binary_string, bp);
+        bp = bin_append_posn(ecc_codewords[i], 4, binary_string, bp);
     }
 
     for (i = 0; i < 28; i += 2) {
