@@ -1,7 +1,7 @@
 /* code1.c - USS Code One */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -30,6 +30,7 @@
  */
 /* SPDX-License-Identifier: BSD-3-Clause */
 
+#include <assert.h>
 #include <math.h>
 #include <stdio.h>
 #include "common.h"
@@ -37,6 +38,17 @@
 #include "reedsol.h"
 #include "large.h"
 
+#define C1_MAX_CWS      1480 /* Max data codewords for Version H */
+#define C1_MAX_CWS_S    "1480" /* String version of above */
+#define C1_MAX_ECCS     560 /* Max ECC codewords for Version H */
+
+#define C1_ASCII    1
+#define C1_C40      2
+#define C1_DECIMAL  3
+#define C1_TEXT     4
+#define C1_EDI      5
+#define C1_BYTE     6
+
 /* Add solid bar */
 static void c1_horiz(struct zint_symbol *symbol, const int row_no, const int full) {
     int i;
@@ -236,7 +248,7 @@ static int c1_look_ahead_test(const unsigned char source[], const int length, co
         }
 
         /* Step P */
-        if (gs1 && (c == '[')) {
+        if (gs1 && c == '\x1D') {
             byte_count += C1_MULT_3; /* Step P1 */
         } else {
             byte_count += C1_MULT_1; /* Step P2 */
@@ -443,11 +455,15 @@ static int c1_codewords_remaining(struct zint_symbol *symbol, const int tp) {
 static int c1_c40text_cnt(const int current_mode, const int gs1, unsigned char input) {
     int cnt;
 
-    if (gs1 && input == '[') {
+    if (gs1 && input == '\x1D') {
         return 2;
     }
     cnt = 1;
-    if ((current_mode == C1_C40 && c40_shift[input]) || (current_mode == C1_TEXT && text_shift[input])) {
+    if (input & 0x80) {
+        cnt += 2;
+        input -= 128;
+    }
+    if ((current_mode == C1_C40 && c1_c40_shift[input]) || (current_mode == C1_TEXT && c1_text_shift[input])) {
         cnt += 1;
     }
 
@@ -473,7 +489,7 @@ static void c1_eci_escape(const int eci, unsigned char source[], const int lengt
 /* Convert to codewords */
 static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int length, const int eci,
             const int seg_count, const int gs1, unsigned int target[], int *p_tp, int *p_last_mode) {
-    int current_mode, next_mode;
+    int current_mode, next_mode, last_mode;
     int sp = 0;
     int tp = *p_tp;
     int i;
@@ -497,14 +513,14 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
         if (length >= 15 && num_digits[0] >= 15) {
             target[tp++] = 236; /* FNC1 and change to Decimal */
             next_mode = C1_DECIMAL;
-            if (debug_print) printf("FNC1Dec ");
+            if (debug_print) fputs("FNC1Dec ", stdout);
         } else if (length >= 7 && num_digits[0] == length) {
             target[tp++] = 236; /* FNC1 and change to Decimal */
             next_mode = C1_DECIMAL;
-            if (debug_print) printf("FNC1Dec ");
+            if (debug_print) fputs("FNC1Dec ", stdout);
         } else {
             target[tp++] = 232; /* FNC1 */
-            if (debug_print) printf("FNC1 ");
+            if (debug_print) fputs("FNC1 ", stdout);
         }
         /* Note ignoring Structured Append and ECI if GS1 mode (up to caller to warn/error) */
     } else {
@@ -516,11 +532,11 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
                     target[tp++] = 233; /* FNC2 */
                     target[tp++] = (symbol->structapp.index - 1) * 15 + (symbol->structapp.count - 1);
                     target[tp++] = '\\' + 1; /* Escape char */
-                    if (debug_print) printf("SAGrp ");
+                    if (debug_print) fputs("SAGrp ", stdout);
                 } else {
                     target[tp++] = (symbol->structapp.index - 1) * 15 + (symbol->structapp.count - 1);
                     target[tp++] = 233; /* FNC2 */
-                    if (debug_print) printf("FNC2 ");
+                    if (debug_print) fputs("FNC2 ", stdout);
                 }
             } else { /* Extended Group mode */
                 if ((eci || seg_count > 1) && symbol->structapp.index == 1) {
@@ -530,12 +546,12 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
                     target[tp++] = 233; /* FNC2 */
                     target[tp++] = symbol->structapp.index;
                     target[tp++] = symbol->structapp.count;
-                    if (debug_print) printf("SAExGrp ");
+                    if (debug_print) fputs("SAExGrp ", stdout);
                 } else {
                     target[tp++] = symbol->structapp.index;
                     target[tp++] = symbol->structapp.count;
                     target[tp++] = 233; /* FNC2 */
-                    if (debug_print) printf("FNC2 ");
+                    if (debug_print) fputs("FNC2 ", stdout);
                 }
             }
             if (eci) {
@@ -547,7 +563,7 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
             if (tp == 0) {
                 target[tp++] = 129; /* Pad */
                 target[tp++] = '\\' + 1; /* Escape char */
-                if (debug_print) printf("PADEsc ");
+                if (debug_print) fputs("PADEsc ", stdout);
             }
             if (eci) {
                 c1_eci_escape(eci, source, length, eci_buf, eci_length);
@@ -559,20 +575,21 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
     }
 
     do {
+        last_mode = current_mode;
         if (current_mode != next_mode) {
             /* Change mode */
             switch (next_mode) {
                 case C1_C40: target[tp++] = 230;
-                    if (debug_print) printf("->C40 ");
+                    if (debug_print) fputs("->C40 ", stdout);
                     break;
                 case C1_TEXT: target[tp++] = 239;
-                    if (debug_print) printf("->Text ");
+                    if (debug_print) fputs("->Text ", stdout);
                     break;
                 case C1_EDI: target[tp++] = 238;
-                    if (debug_print) printf("->EDI ");
+                    if (debug_print) fputs("->EDI ", stdout);
                     break;
                 case C1_BYTE: target[tp++] = 231;
-                    if (debug_print) printf("->Byte ");
+                    if (debug_print) fputs("->Byte ", stdout);
                     byte_start = tp;
                     target[tp++] = 0; /* Byte count holder (may be expanded to 2 codewords) */
                     break;
@@ -601,17 +618,17 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
                     if (debug_print) printf("ASCDD(%.2s) ", source + sp);
                     sp += 2;
                 } else {
-                    if ((gs1) && (source[sp] == '[')) {
+                    if (gs1 && source[sp] == '\x1D') {
                         if (length - (sp + 1) >= 15 && num_digits[sp + 1] >= 15) {
                             /* Step B4 */
                             target[tp++] = 236; /* FNC1 and change to Decimal */
-                            if (debug_print) printf("FNC1 ");
+                            if (debug_print) fputs("FNC1 ", stdout);
                             sp++;
                             next_mode = C1_DECIMAL;
                         } else if (length - (sp + 1) >= 7 && num_digits[sp + 1] == length - (sp + 1)) {
                             /* Step B5 */
                             target[tp++] = 236; /* FNC1 and change to Decimal */
-                            if (debug_print) printf("FNC1 ");
+                            if (debug_print) fputs("FNC1 ", stdout);
                             sp++;
                             next_mode = C1_DECIMAL;
                         }
@@ -621,6 +638,9 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
 
                         /* Step B6 */
                         next_mode = c1_look_ahead_test(source, length, sp, current_mode, gs1);
+                        if (next_mode == last_mode) { /* Avoid looping on latch (ticket #300 (#8) Andre Maute) */
+                            next_mode = C1_ASCII;
+                        }
 
                         if (next_mode == C1_ASCII) {
                             if (debug_print) printf("ASC(%d) ", source[sp]);
@@ -630,10 +650,10 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
                                 target[tp++] = 235; /* FNC4 (Upper Shift) */
                                 target[tp++] = (source[sp] - 128) + 1;
                                 if (debug_print) printf("UpSh(%d) ", source[sp]);
-                            } else if ((gs1) && (source[sp] == '[')) {
+                            } else if (gs1 && source[sp] == '\x1D') {
                                 /* Step B8 */
                                 target[tp++] = 232; /* FNC1 */
-                                if (debug_print) printf("FNC1 ");
+                                if (debug_print) fputs("FNC1 ", stdout);
                             } else {
                                 /* Step B8 */
                                 target[tp++] = source[sp] + 1;
@@ -665,19 +685,19 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
             if (next_mode != current_mode) {
                 /* Step C/D1c */
                 target[tp++] = 255; /* Unlatch */
-                if (debug_print) printf("Unlatch ");
+                if (debug_print) fputs("Unlatch ", stdout);
             } else {
                 /* Step C/D2 */
                 const char *ct_shift, *ct_value;
 
                 if (current_mode == C1_C40) {
-                    ct_shift = c40_shift;
-                    ct_value = c40_value;
+                    ct_shift = c1_c40_shift;
+                    ct_value = c1_c40_value;
                 } else {
-                    ct_shift = text_shift;
-                    ct_value = text_value;
+                    ct_shift = c1_text_shift;
+                    ct_value = c1_text_value;
                 }
-                if (debug_print) printf(current_mode == C1_C40 ? "C40 " : "TEXT ");
+                if (debug_print) fputs(current_mode == C1_C40 ? "C40 " : "TEXT ", stdout);
 
                 if (source[sp] & 0x80) {
                     cte_buffer[cte_p++] = 1; /* Shift 2 */
@@ -686,7 +706,7 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
                         cte_buffer[cte_p++] = ct_shift[source[sp] - 128] - 1;
                     }
                     cte_buffer[cte_p++] = ct_value[source[sp] - 128];
-                } else if (gs1 && (source[sp] == '[')) {
+                } else if (gs1 && source[sp] == '\x1D') {
                     cte_buffer[cte_p++] = 1; /* Shift 2 */
                     cte_buffer[cte_p++] = 27; /* FNC1 */
                 } else {
@@ -727,13 +747,13 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
                     /* No unlatch needed if data fits as ASCII in last data codeword */
                 } else {
                     target[tp++] = 255; /* Unlatch */
-                    if (debug_print) printf("Unlatch ");
+                    if (debug_print) fputs("Unlatch ", stdout);
                 }
             } else {
                 /* Step E2 */
                 static const char edi_nonalphanum_chars[] = "\015*> ";
 
-                if (debug_print) printf("EDI ");
+                if (debug_print) fputs("EDI ", stdout);
 
                 if (z_isdigit(source[sp])) {
                     cte_buffer[cte_p++] = source[sp] - '0' + 4;
@@ -752,7 +772,7 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
         } else if (current_mode == C1_DECIMAL) {
             /* Step F - Decimal encodation */
 
-            if (debug_print) printf("DEC ");
+            if (debug_print) fputs("DEC ", stdout);
 
             next_mode = C1_DECIMAL;
 
@@ -808,7 +828,7 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
         } else if (current_mode == C1_BYTE) {
             next_mode = C1_BYTE;
 
-            if (gs1 && (source[sp] == '[')) {
+            if (gs1 && source[sp] == '\x1D') {
                 next_mode = C1_ASCII;
             } else {
                 if (source[sp] <= 127) {
@@ -836,8 +856,8 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
             }
         }
 
-        if (tp > 1480) {
-            if (debug_print) printf("\n");
+        if (tp > C1_MAX_CWS) {
+            if (debug_print) fputc('\n', stdout);
             /* Data is too large for symbol */
             return 0;
         }
@@ -883,7 +903,7 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
                     } else if (source[sp] & 0x80) {
                         target[tp++] = 235; /* FNC4 (Upper Shift) */
                         target[tp++] = (source[sp] - 128) + 1;
-                    } else if ((gs1) && (source[sp] == '[')) {
+                    } else if (gs1 && source[sp] == '\x1D') {
                         target[tp++] = 232; /* FNC1 */
                     } else {
                         target[tp++] = source[sp] + 1;
@@ -938,7 +958,7 @@ static int c1_encode(struct zint_symbol *symbol, unsigned char source[], int len
     }
 
     /* Re-check length of data */
-    if (tp > 1480) {
+    if (tp > C1_MAX_CWS) {
         /* Data is too large for symbol */
         return 0;
     }
@@ -1005,56 +1025,53 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
     int row, col;
     int sub_version = 0;
     rs_t rs;
-    int error_number = 0;
     const int gs1 = (symbol->input_mode & 0x07) == GS1_MODE;
     const int debug_print = symbol->debug & ZINT_DEBUG_PRINT;
 
     if ((symbol->option_2 < 0) || (symbol->option_2 > 10)) {
-        strcpy(symbol->errtxt, "513: Invalid symbol size");
-        return ZINT_ERROR_INVALID_OPTION;
+        return errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 513, "Version '%d' out of range (1 to 10)",
+                        symbol->option_2);
     }
 
     if (symbol->structapp.count) {
         if (symbol->option_2 == 9) { /* Version S */
-            strcpy(symbol->errtxt, "714: Structured Append not available for Version S");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 714, "Structured Append not available for Version S");
         }
         if (gs1) {
-            strcpy(symbol->errtxt, "710: Cannot have Structured Append and GS1 mode at the same time");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 710,
+                            "Cannot have Structured Append and GS1 mode at the same time");
         }
         if (symbol->structapp.count < 2 || symbol->structapp.count > 128) {
-            strcpy(symbol->errtxt, "711: Structured Append count out of range (2-128)");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 711,
+                            "Structured Append count '%d' out of range (2 to 128)", symbol->structapp.count);
         }
         if (symbol->structapp.index < 1 || symbol->structapp.index > symbol->structapp.count) {
-            sprintf(symbol->errtxt, "712: Structured Append index out of range (1-%d)", symbol->structapp.count);
-            return ZINT_ERROR_INVALID_OPTION;
+            return ZEXT errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 712,
+                                "Structured Append index '%1$d' out of range (1 to count %2$d)",
+                                symbol->structapp.index, symbol->structapp.count);
         }
         if (symbol->structapp.id[0]) {
-            strcpy(symbol->errtxt, "713: Structured Append ID not available for Code One");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 713, "Structured Append ID not available for Code One");
         }
     }
 
     if (symbol->option_2 == 9) {
         /* Version S */
         int codewords;
-        large_int elreg;
+        large_uint elreg;
         unsigned int target[30], ecc[15];
         int block_width;
 
         if (seg_count > 1) {
-            strcpy(symbol->errtxt, "715: Multiple segments not supported for Version S");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 715, "Multiple segments not supported for Version S");
         }
         if (segs[0].length > 18) {
-            strcpy(symbol->errtxt, "514: Input data too long for Version S");
-            return ZINT_ERROR_TOO_LONG;
+            return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 514, "Input length %d too long for Version S (maximum 18)",
+                            segs[0].length);
         }
-        if (!is_sane(NEON_F, segs[0].source, segs[0].length)) {
-            strcpy(symbol->errtxt, "515: Invalid input data (Version S encodes numeric input only)");
-            return ZINT_ERROR_INVALID_DATA;
+        if ((i = not_sane(NEON_F, segs[0].source, segs[0].length))) {
+            return errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 515,
+                            "Invalid character at position %d in input (Version S encodes digits only)", i);
         }
 
         size = 9;
@@ -1089,13 +1106,13 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
         rs_encode_uint(&rs, codewords, target, ecc);
 
         for (i = 0; i < codewords; i++) {
-            target[i + codewords] = ecc[codewords - i - 1];
+            target[i + codewords] = ecc[i];
         }
 
         if (debug_print) {
             printf("Codewords (%d): ", codewords);
             for (i = 0; i < codewords * 2; i++) printf(" %d", (int) target[i]);
-            printf("\n");
+            fputc('\n', stdout);
         }
 
         i = 0;
@@ -1120,22 +1137,23 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
 
     } else if (symbol->option_2 == 10) {
         /* Version T */
-        unsigned int target[90 + 2]; /* Allow for 90 BYTE mode (+ latch and byte count) */
+        unsigned int target[C1_MAX_CWS + C1_MAX_ECCS]; /* Use same buffer size as A to H to avail of loop checks */
         unsigned int ecc[22];
         int data_length;
         int data_cw, ecc_cw, block_width;
-        int last_mode;
+        int last_mode = 0; /* Suppress gcc 14 "-Wmaybe-uninitialized" false positive */
 
-        if (c1_total_length_segs(segs, seg_count) > 90) {
-            strcpy(symbol->errtxt, "519: Input data too long for Version T");
-            return ZINT_ERROR_TOO_LONG;
+        if ((i = c1_total_length_segs(segs, seg_count)) > 90) {
+            return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 519, "Input length %d too long for Version T (maximum 90)",
+                            i);
         }
 
         data_length = c1_encode_segs(symbol, segs, seg_count, gs1, target, &last_mode);
 
-        if (data_length == 0 || data_length > 38) {
-            strcpy(symbol->errtxt, "516: Input data too long for Version T");
-            return ZINT_ERROR_TOO_LONG;
+        assert(data_length); /* Can't exceed C1_MAX_CWS as input <= 90 */
+        if (data_length > 38) {
+            return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 516,
+                            "Input too long for Version T, requires %d codewords (maximum 38)", data_length);
         }
 
         size = 10;
@@ -1177,13 +1195,13 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
         rs_encode_uint(&rs, data_cw, target, ecc);
 
         for (i = 0; i < ecc_cw; i++) {
-            target[data_cw + i] = ecc[ecc_cw - i - 1];
+            target[data_cw + i] = ecc[i];
         }
 
         if (debug_print) {
             printf("Codewords (%d):", data_cw + ecc_cw);
             for (i = 0; i < data_cw + ecc_cw; i++) printf(" %d", (int) target[i]);
-            printf("\n");
+            fputc('\n', stdout);
         }
 
         i = 0;
@@ -1206,7 +1224,7 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
 
     } else {
         /* Versions A to H */
-        unsigned int target[1480 + 560];
+        unsigned int target[C1_MAX_CWS + C1_MAX_ECCS];
         unsigned int sub_data[185], sub_ecc[70];
         int data_length;
         int data_cw;
@@ -1216,8 +1234,8 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
         data_length = c1_encode_segs(symbol, segs, seg_count, gs1, target, &last_mode);
 
         if (data_length == 0) {
-            strcpy(symbol->errtxt, "517: Input data is too long");
-            return ZINT_ERROR_TOO_LONG;
+            return errtxt(ZINT_ERROR_TOO_LONG, symbol, 517,
+                            "Input too long, requires too many codewords (maximum " C1_MAX_CWS_S ")");
         }
 
         for (i = 7; i >= 0; i--) {
@@ -1231,8 +1249,9 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
         }
 
         if ((symbol->option_2 != 0) && (symbol->option_2 < size)) {
-            strcpy(symbol->errtxt, "518: Input too long for selected symbol size");
-            return ZINT_ERROR_TOO_LONG;
+            return ZEXT errtxtf(ZINT_ERROR_TOO_LONG, symbol, 518,
+                                "Input too long for Version %1$c, requires %2$d codewords (maximum %3$d)",
+                                'A' + symbol->option_2 - 1, data_length, c1_data_length[symbol->option_2 - 1]);
         }
         data_cw = c1_data_length[size - 1];
 
@@ -1249,7 +1268,7 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
                 target[i] = 129; /* Pad */
             }
         } else if (debug_print) {
-            printf("No padding\n");
+            fputs("No padding\n", stdout);
         }
 
         /* Calculate error correction data */
@@ -1266,14 +1285,14 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
             }
             rs_encode_uint(&rs, data_blocks, sub_data, sub_ecc);
             for (j = 0; j < ecc_blocks; j++) {
-                target[data_cw + j * blocks + i] = sub_ecc[ecc_blocks - 1 - j];
+                target[data_cw + j * blocks + i] = sub_ecc[j];
             }
         }
 
         if (debug_print) {
             printf("Codewords (%d):", data_cw + ecc_length);
             for (i = 0; i < data_cw + ecc_length; i++) printf(" %d", (int) target[i]);
-            printf("\n");
+            fputc('\n', stdout);
         }
 
         i = 0;
@@ -1559,16 +1578,14 @@ INTERNAL int codeone(struct zint_symbol *symbol, struct zint_seg segs[], const i
 
     if (symbol->option_2 == 9) { /* Version S */
         if (symbol->eci || gs1) {
-            sprintf(symbol->errtxt, "511: %s ignored for Version S",
-                    symbol->eci && gs1 ? "ECI and GS1 mode" : symbol->eci ? "ECI" : "GS1 mode");
-            error_number = ZINT_WARN_INVALID_OPTION;
+            return errtxtf(ZINT_WARN_INVALID_OPTION, symbol, 511, "%s ignored for Version S",
+                            symbol->eci && gs1 ? "ECI and GS1 mode" : symbol->eci ? "ECI" : "GS1 mode");
         }
     } else if (symbol->eci && gs1) {
-        strcpy(symbol->errtxt, "512: ECI ignored for GS1 mode");
-        error_number = ZINT_WARN_INVALID_OPTION;
+        return errtxt(ZINT_WARN_INVALID_OPTION, symbol, 512, "ECI ignored for GS1 mode");
     }
 
-    return error_number;
+    return 0;
 }
 
 /* vim: set ts=4 sw=4 et : */
