@@ -1,7 +1,7 @@
 /* composite.c - Handles GS1 Composite Symbols */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -56,14 +56,13 @@
 #include "gs1.h"
 #include "general_field.h"
 
-#define UINT unsigned short
 #include "composite.h"
 
 INTERNAL int gs1_128_cc(struct zint_symbol *symbol, unsigned char source[], int length, const int cc_mode,
                 const int cc_rows);
 
 INTERNAL int eanx_cc(struct zint_symbol *symbol, unsigned char source[], int length, const int cc_rows);
-INTERNAL int ean_leading_zeroes(struct zint_symbol *symbol, const unsigned char source[],
+INTERNAL int ean_leading_zeroes(struct zint_symbol *symbol, const unsigned char source[], const int length,
                 unsigned char local_source[], int *p_with_addon, unsigned char *zfirst_part,
                 unsigned char *zsecond_part);
 
@@ -71,9 +70,9 @@ INTERNAL int dbar_omnstk_set_height(struct zint_symbol *symbol, const int first_
 INTERNAL int dbar_omn_cc(struct zint_symbol *symbol, unsigned char source[], int length, const int cc_rows);
 INTERNAL int dbar_ltd_cc(struct zint_symbol *symbol, unsigned char source[], int length, const int cc_rows);
 INTERNAL int dbar_exp_cc(struct zint_symbol *symbol, unsigned char source[], int length, const int cc_rows);
-INTERNAL int dbar_date(const unsigned char source[], const int src_posn);
+INTERNAL int dbar_date(const unsigned char source[], const int length, const int src_posn);
 
-static int _min(const int first, const int second) {
+static int cc_min(const int first, const int second) {
 
     if (first <= second)
         return first;
@@ -82,23 +81,23 @@ static int _min(const int first, const int second) {
 }
 
 /* gets bit in bitString at bitPos */
-static int getBit(const UINT *bitStr, const int bitPos) {
+static int cc_getBit(const unsigned short *bitStr, const int bitPos) {
     return !!(bitStr[bitPos >> 4] & (0x8000 >> (bitPos & 15)));
 }
 
 /* converts bit string to base 928 values, codeWords[0] is highest order */
-static int encode928(const UINT bitString[], UINT codeWords[], const int bitLng) {
+static int cc_encode928(const unsigned short bitString[], unsigned short codeWords[], const int bitLng) {
     int i, j, b, cwNdx, cwLng;
     for (cwNdx = cwLng = b = 0; b < bitLng; b += 69, cwNdx += 7) {
-        const int bitCnt = _min(bitLng - b, 69);
+        const int bitCnt = cc_min(bitLng - b, 69);
         int cwCnt;
         cwLng += cwCnt = bitCnt / 10 + 1;
         for (i = 0; i < cwCnt; i++)
             codeWords[cwNdx + i] = 0; /* init 0 */
         for (i = 0; i < bitCnt; i++) {
-            if (getBit(bitString, b + bitCnt - i - 1)) {
+            if (cc_getBit(bitString, b + bitCnt - i - 1)) {
                 for (j = 0; j < cwCnt; j++)
-                    codeWords[cwNdx + j] += pwr928[i][j + 7 - cwCnt];
+                    codeWords[cwNdx + j] += cc_pwr928[i][j + 7 - cwCnt];
             }
         }
         for (i = cwCnt - 1; i > 0; i--) {
@@ -117,8 +116,8 @@ static void cc_a(struct zint_symbol *symbol, const char source[], const int cc_w
     int LeftRAPStart, RightRAPStart, CentreRAPStart, StartCluster;
     int LeftRAP, RightRAP, CentreRAP, Cluster;
     int loop;
-    UINT codeWords[28] = {0};
-    UINT bitStr[13] = {0};
+    unsigned short codeWords[28] = {0};
+    unsigned short bitStr[13] = {0};
     char pattern[580];
     int bp = 0;
     const int debug_print = symbol->debug & ZINT_DEBUG_PRINT;
@@ -140,7 +139,7 @@ static void cc_a(struct zint_symbol *symbol, const char source[], const int cc_w
     }
 
     /* encode codeWords from bitStr */
-    cwCnt = encode928(bitStr, codeWords, bitlen);
+    cwCnt = cc_encode928(bitStr, codeWords, bitlen);
 
     switch (cc_width) {
         case 2:
@@ -191,9 +190,9 @@ static void cc_a(struct zint_symbol *symbol, const char source[], const int cc_w
             break;
     }
 
-    rows = ccaVariants[variant];
-    k = ccaVariants[17 + variant];
-    offset = ccaVariants[34 + variant];
+    rows = cc_aVariants[variant];
+    k = cc_aVariants[17 + variant];
+    offset = cc_aVariants[34 + variant];
 
     /* Reed-Solomon error correction */
 
@@ -201,9 +200,9 @@ static void cc_a(struct zint_symbol *symbol, const char source[], const int cc_w
         total = (codeWords[i] + rsCodeWords[k - 1]) % 929;
         for (j = k - 1; j >= 0; j--) {
             if (j == 0) {
-                rsCodeWords[j] = (929 - (total * ccaCoeffs[offset + j]) % 929) % 929;
+                rsCodeWords[j] = (929 - (total * cc_aCoeffs[offset + j]) % 929) % 929;
             } else {
-                rsCodeWords[j] = (rsCodeWords[j - 1] + 929 - (total * ccaCoeffs[offset + j]) % 929) % 929;
+                rsCodeWords[j] = (rsCodeWords[j - 1] + 929 - (total * cc_aCoeffs[offset + j]) % 929) % 929;
             }
         }
     }
@@ -220,10 +219,10 @@ static void cc_a(struct zint_symbol *symbol, const char source[], const int cc_w
     }
 
     /* Place data into table */
-    LeftRAPStart = aRAPTable[variant];
-    CentreRAPStart = aRAPTable[variant + 17];
-    RightRAPStart = aRAPTable[variant + 34];
-    StartCluster = aRAPTable[variant + 51] / 3;
+    LeftRAPStart = cc_aRAPTable[variant];
+    CentreRAPStart = cc_aRAPTable[variant + 17];
+    RightRAPStart = cc_aRAPTable[variant + 34];
+    StartCluster = cc_aRAPTable[variant + 51] / 3;
 
     LeftRAP = LeftRAPStart;
     CentreRAP = CentreRAPStart;
@@ -301,7 +300,8 @@ static void cc_b(struct zint_symbol *symbol, const char source[], const int cc_w
     const int length = (int) strlen(source) / 8;
     int i;
     unsigned char *data_string = (unsigned char *) z_alloca(length + 3);
-    int chainemc[180], mclength = 0;
+    short chainemc[180];
+    int mclength = 0;
     int k, j, p, longueur, mccorrection[50] = {0}, offset;
     int total;
     char pattern[580];
@@ -325,7 +325,7 @@ static void cc_b(struct zint_symbol *symbol, const char source[], const int cc_w
     /* "the CC-B component shall have codeword 920 in the first symbol character position" (section 9a) */
     chainemc[mclength++] = 920;
 
-    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, 0, debug_print);
+    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, 0);
 
     /* Now figure out which variant of the symbol to use and load values accordingly */
 
@@ -517,7 +517,8 @@ static void cc_c(struct zint_symbol *symbol, const char source[], const int cc_w
     const int length = (int) strlen(source) / 8;
     int i, p;
     unsigned char *data_string = (unsigned char *) z_alloca(length + 4);
-    int chainemc[1000], mclength = 0, k;
+    short chainemc[1000];
+    int mclength = 0, k;
     int offset, longueur, loop, total, j, mccorrection[520] = {0};
     int c1, c2, c3, dummy[35];
     char pattern[580];
@@ -538,14 +539,14 @@ static void cc_c(struct zint_symbol *symbol, const char source[], const int cc_w
     chainemc[mclength++] = 0; /* space for length descriptor */
     chainemc[mclength++] = 920; /* CC-C identifier */
 
-    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, 0, debug_print);
+    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, 0);
 
     chainemc[0] = mclength;
 
     if (debug_print) {
         printf("CC-C Codewords (%d):", mclength);
         for (i = 0; i < mclength; i++) printf(" %d", chainemc[i]);
-        printf("\n");
+        fputc('\n', stdout);
     }
 
     k = 1;
@@ -650,7 +651,7 @@ static void cc_c(struct zint_symbol *symbol, const char source[], const int cc_w
     }
 }
 
-static int calc_padding_cca(const int binary_length, const int cc_width) {
+static int cc_a_calc_padding(const int binary_length, const int cc_width) {
     int target_bitsize = 0;
 
     switch (cc_width) {
@@ -702,7 +703,7 @@ static int calc_padding_cca(const int binary_length, const int cc_width) {
     return target_bitsize;
 }
 
-static int calc_padding_ccb(const int binary_length, const int cc_width) {
+static int cc_b_calc_padding(const int binary_length, const int cc_width) {
     int target_bitsize = 0;
 
     switch (cc_width) {
@@ -776,7 +777,7 @@ static int calc_padding_ccb(const int binary_length, const int cc_width) {
     return target_bitsize;
 }
 
-static int calc_padding_ccc(const int binary_length, int *cc_width, const int linear_width, int *ecc) {
+static int cc_c_calc_padding(const int binary_length, int *p_cc_width, const int linear_width, int *p_ecc_level) {
     int target_bitsize = 0;
     int byte_length, codewords_used, ecc_level, ecc_codewords, rows;
     int codewords_total, target_codewords, target_bytesize;
@@ -804,22 +805,25 @@ static int calc_padding_ccc(const int binary_length, int *cc_width, const int li
     } else {
         return 0;
     }
-    *(ecc) = ecc_level;
+    *p_ecc_level = ecc_level;
     ecc_codewords = 1 << (ecc_level + 1);
 
     codewords_used += ecc_codewords;
     codewords_used += 3;
 
+    /* Minimum possible linear width (with GS1_NO_CHECK) is 11*5 (start, FNC1, linkage, data, check) + 13 stop */
+    assert(linear_width >= 68);
     /* -52 = 7 left shift (section 12.3 f) + 10 right quiet zone - 17 start + 2x17 row indicators + 18 stop */
-    *(cc_width) = (linear_width - 52) / 17;
-    if (*(cc_width) > 30) {
-        *(cc_width) = 30;
+    *p_cc_width = linear_width == 68 ? 1 : (linear_width - 52) / 17; /* Ensure > 0 */
+    if (*p_cc_width > 30) {
+        *p_cc_width = 30;
     }
-    rows = (int) ceil((double) codewords_used / *(cc_width));
+    assert(*p_cc_width > 0);
+    rows = (int) ceil((double) codewords_used / *p_cc_width);
     /* stop the symbol from becoming too high */
-    while (rows > 30 && *(cc_width) < 30) {
-        *(cc_width) = *(cc_width) + 1;
-        rows = (int) ceil((double) codewords_used / *(cc_width));
+    while (rows > 30 && *p_cc_width < 30) {
+        (*p_cc_width)++;
+        rows = (int) ceil((double) codewords_used / *p_cc_width);
     }
 
     if (rows > 30) { /* Should never happen given `codewords_used` check above (865 / 30 ~ 28.83) */
@@ -829,7 +833,7 @@ static int calc_padding_ccc(const int binary_length, int *cc_width, const int li
         rows = 3;
     }
 
-    codewords_total = *(cc_width) * rows;
+    codewords_total = *p_cc_width * rows;
 
     target_codewords = codewords_total - ecc_codewords;
     target_codewords -= 3;
@@ -843,14 +847,14 @@ static int calc_padding_ccc(const int binary_length, int *cc_width, const int li
 }
 
 /* Handles all data encodation from section 5 of ISO/IEC 24723 */
-static int cc_binary_string(struct zint_symbol *symbol, const unsigned char source[], const int source_len,
-            char binary_string[], const int cc_mode, int *cc_width, int *ecc, const int linear_width) {
+static int cc_binary_string(struct zint_symbol *symbol, const unsigned char source[], const int length,
+            char binary_string[], const int cc_mode, int *p_cc_width, int *p_ecc_level, const int linear_width) {
     int encoding_method, read_posn, alpha_pad;
     int i, j, ai_crop, ai_crop_posn, fnc1_latch;
     int ai90_mode, remainder;
     char last_digit = '\0';
     int mode;
-    char *general_field = (char *) z_alloca(source_len + 1);
+    char *general_field = (char *) z_alloca(length + 1);
     int target_bitsize;
     int bp = 0;
     const int debug_print = symbol->debug & ZINT_DEBUG_PRINT;
@@ -861,16 +865,16 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
     ai_crop_posn = -1;
     fnc1_latch = 0;
     alpha_pad = 0;
-    *ecc = 0;
+    *p_ecc_level = 0;
     target_bitsize = 0;
     mode = NUMERIC;
 
-    if ((source[0] == '1') && ((source[1] == '0') || (source[1] == '1') || (source[1] == '7'))) {
+    if (length > 1 && (source[0] == '1') && ((source[1] == '0') || (source[1] == '1') || (source[1] == '7'))) {
         /* Source starts (10), (11) or (17) */
-        if (source[1] == '0' || dbar_date(source, 2) >= 0) { /* Check date valid if (11) or (17) */
+        if (source[1] == '0' || dbar_date(source, length, 2) >= 0) { /* Check date valid if (11) or (17) */
             encoding_method = 2;
         }
-    } else if ((source[0] == '9') && (source[1] == '0')) {
+    } else if (length > 1 && (source[0] == '9') && (source[1] == '0')) {
         /* Source starts (90) */
         encoding_method = 3;
     }
@@ -890,8 +894,9 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
             read_posn = 2;
         } else {
             /* Production Date (11) or Expiration Date (17) */
+            assert(length >= 8); /* Due to `dbar_date()` check above */
 
-            bp = bin_append_posn(dbar_date(source, 2), 16, binary_string, bp);
+            bp = bin_append_posn(dbar_date(source, length, 2), 16, binary_string, bp);
 
             if (source[1] == '1') {
                 /* Production Date AI 11 */
@@ -902,7 +907,7 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
             }
             read_posn = 8;
 
-            if ((source[read_posn] == '1') && (source[read_posn + 1] == '0')) {
+            if (read_posn + 1 < length && (source[read_posn] == '1') && (source[read_posn + 1] == '0')) {
                 /* Followed by AI 10 - strip this from general field */
                 read_posn += 2;
             } else if (source[read_posn]) {
@@ -925,20 +930,21 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
 
     } else if (encoding_method == 3) {
         /* Encodation Method field of "11" - AI 90 */
-        char *ninety = (char *) z_alloca(source_len + 1);
-        int ninety_len, alpha, alphanum, numeric, test1, test2, test3;
+        unsigned char *ninety = (unsigned char *) z_alloca(length + 1);
+        int ninety_len, alpha, alphanum, numeric, alpha_posn;
 
         /* "This encodation method may be used if an element string with an AI
         90 occurs at the start of the data message, and if the data field
         following the two-digit AI 90 starts with an alphanumeric string which
         complies with a specific format." (para 5.3.2) */
 
-        memset(ninety, 0, source_len + 1);
         i = 0;
-        do {
-            ninety[i] = source[i + 2];
-            i++;
-        } while ((source_len > i + 2) && ('[' != source[i + 2]));
+        if (length > 2) {
+            do {
+                ninety[i] = source[i + 2];
+                i++;
+            } while (i + 2 < length && source[i + 2] != '\x1D');
+        }
         ninety[i] = '\0';
         ninety_len = i;
 
@@ -962,36 +968,27 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
         }
 
         /* must start with 0, 1, 2 or 3 digits followed by an uppercase character */
-        test1 = -1;
-        for (i = 3; i >= 0; i--) {
-            if (z_isupper(ninety[i])) {
-                test1 = i;
+        alpha_posn = -1;
+        if (ninety_len && ninety[0] != '0') { /* Leading zeros are not permitted */
+            for (i = 0; i < ninety_len && i < 4; i++) {
+                if (z_isupper(ninety[i])) {
+                    alpha_posn = i;
+                    break;
+                }
+                if (!z_isdigit(ninety[i])) {
+                    break;
+                }
             }
         }
 
-        test2 = 0;
-        for (i = 0; i < test1; i++) {
-            if (!z_isdigit(ninety[i])) {
-                test2 = 1;
-                break;
-            }
-        }
-
-        /* leading zeros are not permitted */
-        test3 = 0;
-        if ((test1 >= 1) && (ninety[0] == '0')) {
-            test3 = 1;
-        }
-
-        if ((test1 != -1) && (test2 != 1) && (test3 == 0)) {
+        if (alpha_posn != -1) {
             int next_ai_posn;
-            char numeric_part[4];
             int numeric_value;
             int table3_letter;
             /* Encodation method "11" can be used */
             bp = bin_append_posn(3, 2, binary_string, bp); /* "11" */
 
-            numeric -= test1;
+            numeric -= alpha_posn;
             alpha--;
 
             /* Decide on numeric, alpha or alphanumeric mode */
@@ -1016,13 +1013,13 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
 
             next_ai_posn = 2 + ninety_len;
 
-            if (next_ai_posn < source_len && source[next_ai_posn] == '[') {
+            if (next_ai_posn < length && source[next_ai_posn] == '\x1D') {
                 /* There are more AIs afterwards */
-                if (next_ai_posn + 2 < source_len
+                if (next_ai_posn + 2 < length
                         && (source[next_ai_posn + 1] == '2') && (source[next_ai_posn + 2] == '1')) {
                     /* AI 21 follows */
                     ai_crop = 1;
-                } else if (next_ai_posn + 4 < source_len
+                } else if (next_ai_posn + 4 < length
                         && (source[next_ai_posn + 1] == '8') && (source[next_ai_posn + 2] == '0')
                         && (source[next_ai_posn + 3] == '0') && (source[next_ai_posn + 4] == '4')) {
                     /* AI 8004 follows */
@@ -1041,20 +1038,11 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
                     break;
             }
 
-            if (test1 == 0) {
-                strcpy(numeric_part, "0");
-            } else {
-                for (i = 0; i < test1; i++) {
-                    numeric_part[i] = ninety[i];
-                }
-                numeric_part[i] = '\0';
-            }
-
-            numeric_value = atoi(numeric_part);
+            numeric_value = alpha_posn ? to_int(ninety, alpha_posn) : 0;
 
             table3_letter = -1;
             if (numeric_value < 31) {
-                table3_letter = posn("BDHIJKLNPQRSTVWZ", ninety[test1]);
+                table3_letter = posn("BDHIJKLNPQRSTVWZ", ninety[alpha_posn]);
             }
 
             if (table3_letter != -1) {
@@ -1071,10 +1059,10 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
                 bp = bin_append_posn(numeric_value, 10, binary_string, bp);
 
                 /* five bit representation of ASCII character */
-                bp = bin_append_posn(ninety[test1] - 65, 5, binary_string, bp);
+                bp = bin_append_posn(ninety[alpha_posn] - 65, 5, binary_string, bp);
             }
 
-            read_posn = test1 + 3;
+            read_posn = alpha_posn + 3; /* +2 for 90 and +1 to go beyond alpha position */
 
             /* Do Alpha mode encoding of the rest of the AI 90 data field here */
             if (ai90_mode == 2) {
@@ -1086,12 +1074,12 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
                     } else if (z_isdigit(source[read_posn])) {
                         bp = bin_append_posn(source[read_posn] + 4, 6, binary_string, bp);
 
-                    } else if (source[read_posn] == '[') {
+                    } else if (source[read_posn] == '\x1D') {
                         bp = bin_append_posn(31, 5, binary_string, bp);
                     }
 
                     read_posn++;
-                } while ((source[read_posn - 1] != '[') && (source[read_posn - 1] != '\0'));
+                } while (source[read_posn - 1] != '\x1D' && source[read_posn - 1] != '\0');
                 alpha_pad = 1; /* This is overwritten if a general field is encoded */
             }
 
@@ -1114,11 +1102,11 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
     if (fnc1_latch == 1) {
         /* Encodation method "10" has been used but it is not followed by
            AI 10, so a FNC1 character needs to be added */
-        general_field[j] = '[';
+        general_field[j] = '\x1D';
         j++;
     }
 
-    for (i = read_posn; i < source_len; i++) {
+    for (i = read_posn; i < length; i++) {
         /* Skip "[21" or "[8004" AIs if encodation method "11" used */
         if (i == ai_crop_posn) {
             i += ai_crop;
@@ -1140,26 +1128,24 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
 
         if (!general_field_encode(general_field, j, &mode, &last_digit, binary_string, &bp)) {
             /* Invalid character in input data */
-            strcpy(symbol->errtxt, "441: Invalid character in input data");
-            return ZINT_ERROR_INVALID_DATA;
+            return errtxt(ZINT_ERROR_INVALID_DATA, symbol, 441, "Invalid character in input (2D component)");
         }
     }
 
     switch (cc_mode) {
         case 1:
-            target_bitsize = calc_padding_cca(bp, *(cc_width));
+            target_bitsize = cc_a_calc_padding(bp, *p_cc_width);
             break;
         case 2:
-            target_bitsize = calc_padding_ccb(bp, *(cc_width));
+            target_bitsize = cc_b_calc_padding(bp, *p_cc_width);
             break;
         case 3:
-            target_bitsize = calc_padding_ccc(bp, cc_width, linear_width, ecc);
+            target_bitsize = cc_c_calc_padding(bp, p_cc_width, linear_width, p_ecc_level);
             break;
     }
 
     if (target_bitsize == 0) {
-        strcpy(symbol->errtxt, "442: Input too long for selected 2D component");
-        return ZINT_ERROR_TOO_LONG;
+        return errtxt(ZINT_ERROR_TOO_LONG, symbol, 442, "Input too long (2D component)");
     }
 
     remainder = target_bitsize - bp;
@@ -1184,19 +1170,18 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
 
     switch (cc_mode) {
         case 1:
-            target_bitsize = calc_padding_cca(bp, *(cc_width));
+            target_bitsize = cc_a_calc_padding(bp, *p_cc_width);
             break;
         case 2:
-            target_bitsize = calc_padding_ccb(bp, *(cc_width));
+            target_bitsize = cc_b_calc_padding(bp, *p_cc_width);
             break;
         case 3:
-            target_bitsize = calc_padding_ccc(bp, cc_width, linear_width, ecc);
+            target_bitsize = cc_c_calc_padding(bp, p_cc_width, linear_width, p_ecc_level);
             break;
     }
 
     if (target_bitsize == 0) {
-        strcpy(symbol->errtxt, "444: Input too long for selected 2D component");
-        return ZINT_ERROR_TOO_LONG;
+        return errtxt(ZINT_ERROR_TOO_LONG, symbol, 444, "Input too long (2D component)");
     }
 
     if (bp < target_bitsize) {
@@ -1217,7 +1202,7 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
     binary_string[target_bitsize] = '\0';
 
     if (debug_print) {
-        printf("ECC: %d, CC width %d\n", *ecc, *cc_width);
+        printf("ECC: %d, CC width %d\n", *p_ecc_level, *p_cc_width);
         printf("Binary: %s (%d)\n", binary_string, target_bitsize);
     }
 
@@ -1225,19 +1210,19 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
 }
 
 /* Calculate the width of the linear part (primary) */
-static int linear_dummy_run(int input_mode, unsigned char *source, const int length, const int debug, char *errtxt) {
+static int cc_linear_dummy_run(struct zint_symbol *symbol, unsigned char *source, const int length) {
     struct zint_symbol dummy = {0};
     int error_number;
     int linear_width;
 
     dummy.symbology = BARCODE_GS1_128_CC;
     dummy.option_1 = -1;
-    dummy.input_mode = input_mode;
-    dummy.debug = debug;
+    dummy.input_mode = symbol->input_mode;
+    dummy.debug = symbol->debug;
     error_number = gs1_128_cc(&dummy, source, length, 3 /*cc_mode*/, 0 /*cc_rows*/);
     linear_width = dummy.width;
-    if (error_number >= ZINT_ERROR || (debug & ZINT_DEBUG_TEST)) {
-        strcpy(errtxt, dummy.errtxt);
+    if (error_number >= ZINT_ERROR || (symbol->debug & ZINT_DEBUG_TEST)) {
+        (void) errtxt(0, symbol, -1, dummy.errtxt);
     }
 
     if (error_number >= ZINT_ERROR) {
@@ -1246,10 +1231,8 @@ static int linear_dummy_run(int input_mode, unsigned char *source, const int len
     return linear_width;
 }
 
-static const char in_linear_comp[] = " in linear component";
-
 INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int length) {
-    int error_number, warn_number = 0, cc_mode, cc_width = 0, ecc_level = 0;
+    int error_number, cc_mode, cc_width = 0, ecc_level = 0;
     int j, i, k;
     /* Allow for 8 bits + 5-bit latch per char + 1000 bits overhead/padding */
     const unsigned int bs = 13 * length + 1000 + 1;
@@ -1266,31 +1249,27 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     error_number = 0;
     pri_len = (int) strlen(symbol->primary);
     if (pri_len == 0) {
-        strcpy(symbol->errtxt, "445: No primary (linear) message");
-        return ZINT_ERROR_INVALID_OPTION; /* TODO: change to more appropiate ZINT_ERROR_INVALID_DATA */
+        /* TODO: change to more appropiate ZINT_ERROR_INVALID_DATA */
+        return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 445, "No primary (linear) message");
     }
 
     if (length > 2990) {
-        strcpy(symbol->errtxt, "446: 2D component input data too long");
-        return ZINT_ERROR_TOO_LONG;
+        return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 446,
+                        "2D component input too long, requires %d characters (maximum 2990)", length);
     }
 
     cc_mode = symbol->option_1;
     if ((cc_mode == 3) && (symbol->symbology != BARCODE_GS1_128_CC)) {
         /* CC-C can only be used with a GS1-128 linear part */
-        strcpy(symbol->errtxt, "447: Invalid mode (CC-C only valid with GS1-128 linear component)");
-        return ZINT_ERROR_INVALID_OPTION;
+        return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 447,
+                        "Invalid mode (CC-C only valid with GS1-128 linear component)");
     }
 
     if (symbol->symbology == BARCODE_GS1_128_CC) {
         /* Do a test run of encoding the linear component to establish its width */
-        linear_width = linear_dummy_run(symbol->input_mode, (unsigned char *) symbol->primary, pri_len,
-                                        symbol->debug, symbol->errtxt);
+        linear_width = cc_linear_dummy_run(symbol, (unsigned char *) symbol->primary, pri_len);
         if (linear_width == 0) {
-            if (strlen(symbol->errtxt) + strlen(in_linear_comp) < sizeof(symbol->errtxt)) {
-                strcat(symbol->errtxt, in_linear_comp);
-            }
-            return ZINT_ERROR_INVALID_DATA;
+            return errtxt_adj(ZINT_ERROR_INVALID_DATA, symbol, "%1$s%2$s", " (linear component)");
         }
         if (debug_print) {
             printf("GS1-128 linear width: %d\n", linear_width);
@@ -1304,11 +1283,9 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
                 int padded_pri_len;
                 int with_addon;
                 unsigned char padded_pri[21];
-                if (!ean_leading_zeroes(symbol, (unsigned char *) symbol->primary, padded_pri, &with_addon, NULL,
-                        NULL)) {
-                    sprintf(symbol->errtxt, "448: Input too long (%s) in linear component",
-                            with_addon ? "5 character maximum for add-on" : "13 character maximum");
-                    return ZINT_ERROR_TOO_LONG;
+                if (!ean_leading_zeroes(symbol, (unsigned char *) symbol->primary, pri_len, padded_pri, &with_addon,
+                                        NULL, NULL)) {
+                    return errtxt_adj(ZINT_ERROR_TOO_LONG, symbol, "%1$s%2$s", " (linear component)");
                 }
                 padded_pri_len = (int) ustrlen(padded_pri);
                 if (padded_pri_len <= 7) { /* EAN-8 */
@@ -1332,8 +1309,7 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
                 }
             }
             if (cc_width == 0) {
-                strcpy(symbol->errtxt, "449: Input wrong length in linear component");
-                return ZINT_ERROR_TOO_LONG;
+                return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 449, "Input length %d wrong (linear component)", pri_len);
             }
             break;
         case BARCODE_GS1_128_CC: cc_width = 4;
@@ -1408,6 +1384,7 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     linear->symbology = symbol->symbology;
     linear->input_mode = symbol->input_mode;
     linear->output_options = symbol->output_options;
+    linear->show_hrt = symbol->show_hrt;
     linear->option_2 = symbol->option_2;
     linear->option_3 = symbol->option_3;
     /* If symbol->height given minimum row height will be returned, else default height */
@@ -1454,10 +1431,7 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     }
 
     if (error_number) {
-        strcpy(symbol->errtxt, linear->errtxt);
-        if (strlen(symbol->errtxt) + strlen(in_linear_comp) < sizeof(symbol->errtxt)) {
-            strcat(symbol->errtxt, in_linear_comp);
-        }
+        ZEXT errtxtf(0, symbol, -1, "%1$s%2$s", linear->errtxt, " (linear component)");
         if (error_number >= ZINT_ERROR) {
             ZBarcode_Delete(linear);
             return error_number;
@@ -1472,7 +1446,7 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     switch (symbol->symbology) {
             /* Determine horizontal alignment (according to section 12.3) */
         case BARCODE_EANX_CC:
-            switch (ustrlen(linear->text)) { /* Use zero-padded length */
+            switch (linear->text_length) { /* Use zero-padded length */
                 case 8: /* EAN-8 */
                 case 11: /* EAN-8 + 2 */
                 case 14: /* EAN-8 + 5 */
@@ -1489,7 +1463,8 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
                     break;
             }
             break;
-        case BARCODE_GS1_128_CC: if (cc_mode == 3) {
+        case BARCODE_GS1_128_CC:
+            if (cc_mode == 3) {
                 bottom_shift = 7; /* ISO/IEC 24723:2010 12.3 f) */
             } else {
                 /* ISO/IEC 24723:2010 12.3 g) "GS1-128 components linked to the right quiet zone of the CC-A or CC-B:
@@ -1521,10 +1496,8 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
                 bottom_shift = 9;
             }
             break;
-        case BARCODE_DBAR_EXP_CC: k = 1;
-            while ((!(module_is_set(linear, 1, k - 1))) && module_is_set(linear, 1, k)) {
-                k++;
-            }
+        case BARCODE_DBAR_EXP_CC:
+            for (k = 1; !module_is_set(linear, 1, k - 1) && module_is_set(linear, 1, k); k++);
             top_shift = k;
             break;
         case BARCODE_UPCA_CC: bottom_shift = 2;
@@ -1535,10 +1508,8 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
             break;
         case BARCODE_DBAR_OMNSTK_CC: top_shift = 1;
             break;
-        case BARCODE_DBAR_EXPSTK_CC: k = 1;
-            while ((!(module_is_set(linear, 1, k - 1))) && module_is_set(linear, 1, k)) {
-                k++;
-            }
+        case BARCODE_DBAR_EXPSTK_CC:
+            for (k = 1; !module_is_set(linear, 1, k - 1) && module_is_set(linear, 1, k); k++);
             top_shift = k;
             break;
     }
@@ -1583,10 +1554,19 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     if (symbol->output_options & COMPLIANT_HEIGHT) {
         if (symbol->symbology == BARCODE_DBAR_STK_CC) {
             /* Databar Stacked needs special treatment due to asymmetric rows */
-            warn_number = dbar_omnstk_set_height(symbol, symbol->rows - linear->rows + 1 /*first_row*/);
+            error_number = dbar_omnstk_set_height(symbol, symbol->rows - linear->rows + 1 /*first_row*/);
+        } else if (symbol->symbology == BARCODE_DBAR_EXP_CC || symbol->symbology == BARCODE_DBAR_EXPSTK_CC) {
+            /* If symbol->height given then min row height was returned, else default height */
+            if (error_number == 0) { /* Avoid overwriting any `gs1_verify()` warning */
+                error_number = set_height(symbol, symbol->height ? linear->height : 0.0f,
+                                        symbol->height ? 0.0f : linear->height, 0.0f, 0 /*no_errtxt*/);
+            } else {
+                (void) set_height(symbol, symbol->height ? linear->height : 0.0f,
+                                        symbol->height ? 0.0f : linear->height, 0.0f, 1 /*no_errtxt*/);
+            }
         } else {
             /* If symbol->height given then min row height was returned, else default height */
-            warn_number = set_height(symbol, symbol->height ? linear->height : 0.0f,
+            error_number = set_height(symbol, symbol->height ? linear->height : 0.0f,
                                     symbol->height ? 0.0f : linear->height, 0.0f, 0 /*no_errtxt*/);
         }
     } else {
@@ -1598,11 +1578,11 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
         }
     }
 
-    ustrcpy(symbol->text, linear->text);
+    hrt_cpy_nochk(symbol, linear->text, linear->text_length);
 
     ZBarcode_Delete(linear);
 
-    return error_number ? error_number : warn_number;
+    return error_number;
 }
 
 /* vim: set ts=4 sw=4 et : */
