@@ -1,7 +1,7 @@
 /* dotcode.c - Handles DotCode */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2017-2023 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2017-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -658,8 +658,8 @@ static int dc_encode_message(struct zint_symbol *symbol, const unsigned char sou
                 continue;
             }
 
-            if (dc_datum_c(source, length, position) || (source[position] == '[' && gs1)) {
-                if (source[position] == '[') {
+            if (dc_datum_c(source, length, position) || (gs1 && source[position] == '\x1D')) {
+                if (source[position] == '\x1D') {
                     codeword_array[ap++] = 107; /* FNC1 */
                     position++;
                 } else {
@@ -747,7 +747,7 @@ static int dc_encode_message(struct zint_symbol *symbol, const unsigned char sou
             }
 
             /* Step D2 */
-            if ((source[position] == '[') && gs1) {
+            if (gs1 && source[position] == '\x1D') {
                 codeword_array[ap++] = 107; /* FNC1 */
                 position++;
                 if (debug_print) fputs("D2/1 ", stdout);
@@ -841,7 +841,7 @@ static int dc_encode_message(struct zint_symbol *symbol, const unsigned char sou
             }
 
             /* Step E2 */
-            if ((source[position] == '[') && gs1) {
+            if (gs1 && source[position] == '\x1D') {
                 /* Note: this branch probably never reached as no reason to be in Code Set A for GS1 data */
                 codeword_array[ap++] = 107; /* FNC1 */
                 position++;
@@ -1229,8 +1229,8 @@ INTERNAL int dotcode(struct zint_symbol *symbol, struct zint_seg segs[], const i
     unsigned char *masked_codeword_array;
 
     if (symbol->eci > 811799) {
-        strcpy(symbol->errtxt, "525: Invalid ECI");
-        return ZINT_ERROR_INVALID_OPTION;
+        return errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 525, "ECI code '%d' out of range (0 to 811799)",
+                        symbol->eci);
     }
 
     user_mask = (symbol->option_3 >> 8) & 0x0F; /* User mask is mask + 1, so >= 1 and <= 8 */
@@ -1240,16 +1240,16 @@ INTERNAL int dotcode(struct zint_symbol *symbol, struct zint_seg segs[], const i
 
     if (symbol->structapp.count) {
         if (symbol->structapp.count < 2 || symbol->structapp.count > 35) {
-            strcpy(symbol->errtxt, "730: Structured Append count out of range (2-35)");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 730,
+                            "Structured Append count '%d' out of range (2 to 35)", symbol->structapp.count);
         }
         if (symbol->structapp.index < 1 || symbol->structapp.index > symbol->structapp.count) {
-            sprintf(symbol->errtxt, "731: Structured Append index out of range (1-%d)", symbol->structapp.count);
-            return ZINT_ERROR_INVALID_OPTION;
+            return ZEXT errtxtf(ZINT_ERROR_INVALID_OPTION, symbol, 731,
+                                "Structured Append index '%1$d' out of range (1 to count %2$d)",
+                                symbol->structapp.index, symbol->structapp.count);
         }
         if (symbol->structapp.id[0]) {
-            strcpy(symbol->errtxt, "732: Structured Append ID not available for DotCode");
-            return ZINT_ERROR_INVALID_OPTION;
+            return errtxt(ZINT_ERROR_INVALID_OPTION, symbol, 732, "Structured Append ID not available for DotCode");
         }
     }
 
@@ -1258,14 +1258,14 @@ INTERNAL int dotcode(struct zint_symbol *symbol, struct zint_seg segs[], const i
     if (gs1 && warn_number == 0) {
         for (i = 0; i < seg_count; i++) {
             if (segs[i].eci) {
-                strcpy(symbol->errtxt, "733: Using ECI in GS1 mode not supported by GS1 standards");
-                warn_number = ZINT_WARN_NONCOMPLIANT;
+                warn_number = errtxt(ZINT_WARN_NONCOMPLIANT, symbol, 733,
+                                        "Using ECI in GS1 mode not supported by GS1 standards");
                 break;
             }
         }
         if (warn_number == 0 && symbol->structapp.count) {
-            strcpy(symbol->errtxt, "734: Using Structured Append in GS1 mode not supported by GS1 standards");
-            warn_number = ZINT_WARN_NONCOMPLIANT;
+            warn_number = errtxt(ZINT_WARN_NONCOMPLIANT, symbol, 734,
+                                    "Using Structured Append in GS1 mode not supported by GS1 standards");
         }
     }
 
@@ -1337,20 +1337,21 @@ INTERNAL int dotcode(struct zint_symbol *symbol, struct zint_seg segs[], const i
 
     if ((height > 200) || (width > 200)) {
         if (height > 200 && width > 200) {
-            sprintf(symbol->errtxt, "526: Symbol size %dx%d (WxH) is too large", width, height);
+            ZEXT errtxtf(0, symbol, 526, "Symbol size '%1$dx%2$d' (WxH) is too large", width, height);
         } else {
-            sprintf(symbol->errtxt, "528: Symbol %s %d is too large",
-                    width > 200 ? "width" : "height", width > 200 ? width : height);
+            ZEXT errtxtf(0, symbol, 528, "Symbol %1$s '%2$d' is too large", width > 200 ? "width" : "height",
+                        width > 200 ? width : height);
         }
         return ZINT_ERROR_INVALID_OPTION;
     }
 
     if ((height < 5) || (width < 5)) {
         if (height < 5 && width < 5) { /* Won't happen as if width < 5, min height is 19 */
-            sprintf(symbol->errtxt, "527: Symbol size %dx%d (WxH) is too small", width, height); /* Not reached */
+            ZEXT errtxtf(0, symbol, 527, "Symbol size '%1$dx%2$d' (WxH) is too small", width,
+                        height); /* Not reached */
         } else {
-            sprintf(symbol->errtxt, "529: Symbol %s %d is too small",
-                    width < 5 ? "width" : "height", width < 5 ? width : height);
+            ZEXT errtxtf(0, symbol, 529, "Symbol %1$s '%2$d' is too small", width < 5 ? "width" : "height",
+                        width < 5 ? width : height);
         }
         return ZINT_ERROR_INVALID_OPTION;
     }
