@@ -1,6 +1,6 @@
 /*
     libzint - the open source barcode library
-    Copyright (C) 2019-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2019-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -36,40 +36,53 @@ static void test_hrt(const testCtx *const p_ctx) {
 
     struct item {
         int option_2;
-        char *data;
+        int output_options;
+        const char *data;
         int length;
 
-        char *expected;
+        const char *expected;
     };
     /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
-        /*  0*/ { -1, "1", -1, "01" },
-        /*  1*/ { 3, "1", -1, "01" },
-        /*  2*/ { 3, "12", -1, "12" },
-        /*  3*/ { 4, "123", -1, "123" },
-        /*  4*/ { 5, "123", -1, "0123" },
-        /*  5*/ { 5, "12", -1, "0012" },
-        /*  6*/ { 5, "1", -1, "0001" },
-        /*  7*/ { 5, "1234", -1, "1234" },
-        /*  8*/ { 6, "1234", -1, "01234" },
-        /*  9*/ { 6, "123", -1, "00123" },
-        /* 10*/ { 6, "12", -1, "00012" },
-        /* 11*/ { 6, "1", -1, "00001" },
-        /* 12*/ { 7, "1234", -1, "001234" },
-        /* 13*/ { 7, "12345", -1, "012345" },
-        /* 14*/ { 7, "123456", -1, "123456" },
-        /* 15*/ { 7, "1", -1, "000001" },
-        /* 16*/ { 8, "12345", -1, "0012345" },
-        /* 17*/ { 8, "123456", -1, "0123456" },
-        /* 18*/ { 8, "1234567", -1, "1234567" },
-        /* 19*/ { 8, "12", -1, "0000012" },
-        /* 20*/ { 8, "1", -1, "0000001" },
+    static const struct item data[] = {
+        /*  0*/ { -1, -1, "1", -1, "01" },
+        /*  1*/ { -1, BARCODE_RAW_TEXT, "1", -1, "01" }, /* No difference */
+        /*  2*/ { 3, -1, "1", -1, "01" },
+        /*  3*/ { 3, BARCODE_RAW_TEXT, "1", -1, "01" },
+        /*  4*/ { 3, -1, "12", -1, "12" },
+        /*  5*/ { 3, BARCODE_RAW_TEXT, "12", -1, "12" },
+        /*  6*/ { 4, -1, "123", -1, "123" },
+        /*  7*/ { 4, BARCODE_RAW_TEXT, "123", -1, "123" },
+        /*  8*/ { 5, -1, "123", -1, "0123" },
+        /*  9*/ { 5, BARCODE_RAW_TEXT, "123", -1, "0123" },
+        /* 10*/ { 5, -1, "12", -1, "0012" },
+        /* 11*/ { 5, BARCODE_RAW_TEXT, "12", -1, "0012" },
+        /* 12*/ { 5, -1, "1", -1, "0001" },
+        /* 13*/ { 5, -1, "1234", -1, "1234" },
+        /* 14*/ { 6, -1, "1234", -1, "01234" },
+        /* 15*/ { 6, -1, "123", -1, "00123" },
+        /* 16*/ { 6, -1, "12", -1, "00012" },
+        /* 17*/ { 6, -1, "1", -1, "00001" },
+        /* 18*/ { 7, -1, "1234", -1, "001234" },
+        /* 19*/ { 7, -1, "12345", -1, "012345" },
+        /* 20*/ { 7, -1, "123456", -1, "123456" },
+        /* 21*/ { 7, -1, "1", -1, "000001" },
+        /* 22*/ { 8, -1, "12345", -1, "0012345" },
+        /* 23*/ { 8, BARCODE_RAW_TEXT, "12345", -1, "0012345" },
+        /* 24*/ { 8, -1, "123456", -1, "0123456" },
+        /* 25*/ { 8, BARCODE_RAW_TEXT, "123456", -1, "0123456" },
+        /* 26*/ { 8, -1, "1234567", -1, "1234567" },
+        /* 27*/ { 8, BARCODE_RAW_TEXT, "1234567", -1, "1234567" },
+        /* 28*/ { 8, -1, "12", -1, "0000012" },
+        /* 29*/ { 8, BARCODE_RAW_TEXT, "12", -1, "0000012" },
+        /* 30*/ { 8, -1, "1", -1, "0000001" },
+        /* 31*/ { 8, BARCODE_RAW_TEXT, "1", -1, "0000001" },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i, length, ret;
-    struct zint_symbol *symbol;
+    struct zint_symbol *symbol = NULL;
+    int expected_length;
 
-    testStart("test_hrt");
+    testStartSymbol("test_hrt", &symbol);
 
     for (i = 0; i < data_size; i++) {
 
@@ -78,12 +91,18 @@ static void test_hrt(const testCtx *const p_ctx) {
         symbol = ZBarcode_Create();
         assert_nonnull(symbol, "Symbol not created\n");
 
-        length = testUtilSetSymbol(symbol, BARCODE_CHANNEL, -1 /*input_mode*/, -1 /*eci*/, -1 /*option_1*/, data[i].option_2, -1, -1 /*output_options*/, data[i].data, data[i].length, debug);
+        length = testUtilSetSymbol(symbol, BARCODE_CHANNEL, -1 /*input_mode*/, -1 /*eci*/,
+                    -1 /*option_1*/, data[i].option_2, -1, data[i].output_options,
+                    data[i].data, data[i].length, debug);
+        expected_length = (int) strlen(data[i].expected);
 
-        ret = ZBarcode_Encode(symbol, (unsigned char *) data[i].data, length);
+        ret = ZBarcode_Encode(symbol, TCU(data[i].data), length);
         assert_zero(ret, "i:%d ZBarcode_Encode ret %d != 0 %s\n", i, ret, symbol->errtxt);
 
-        assert_zero(strcmp((char *) symbol->text, data[i].expected), "i:%d strcmp(%s, %s) != 0\n", i, symbol->text, data[i].expected);
+        assert_equal(symbol->text_length, expected_length, "i:%d text_length %d != expected_length %d\n",
+                    i, symbol->text_length, expected_length);
+        assert_zero(strcmp((char *) symbol->text, data[i].expected), "i:%d strcmp(%s, %s) != 0\n",
+                    i, symbol->text, data[i].expected);
 
         ZBarcode_Delete(symbol);
     }
@@ -96,71 +115,74 @@ static void test_input(const testCtx *const p_ctx) {
 
     struct item {
         int option_2;
-        char *data;
+        const char *data;
         int ret;
         int expected_rows;
         int expected_width;
+        const char *expected_errtxt;
     };
     /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
-        /*  0*/ { -1, "0", 0, 1, 19 }, /* < 3 ignored */
-        /*  1*/ { 0, "0", 0, 1, 19, },
-        /*  2*/ { 1, "0", 0, 1, 19, },
-        /*  3*/ { 2, "0", 0, 1, 19, },
-        /*  4*/ { 9, "0", 0, 1, 19, }, /* > 8 ignored */
-        /*  5*/ { -1, "00", 0, 1, 19 },
-        /*  6*/ { 3, "00", 0, 1, 19 },
-        /*  7*/ { -1, "26", 0, 1, 19, },
-        /*  8*/ { 3, "26", 0, 1, 19, },
-        /*  9*/ { 3, "27", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 10*/ { 3, "000", 0, 1, 19, },
-        /* 11*/ { 3, "001", 0, 1, 19, },
-        /* 12*/ { 3, "026", 0, 1, 19, },
-        /* 13*/ { -1, "27", 0, 1, 23 }, /* Channel 4 */
-        /* 14*/ { -1, "026", 0, 1, 23, }, /* Defaults to channel 4 due to length */
-        /* 15*/ { 3, "0026", 0, 1, 19, },
-        /* 16*/ { 3, "1234", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 17*/ { 4, "000", 0, 1, 23 },
-        /* 18*/ { -1, "000", 0, 1, 23 }, /* Defaults to channel 4 due to length */
-        /* 19*/ { 4, "026", 0, 1, 23 },
-        /* 20*/ { 4, "0000026", 0, 1, 23 },
-        /* 21*/ { 4, "0000", 0, 1, 23 },
-        /* 22*/ { 4, "292", 0, 1, 23 },
-        /* 23*/ { 4, "293", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 24*/ { -1, "293", 0, 1, 27 }, /* Channel 5 */
-        /* 25*/ { 5, "0000", 0, 1, 27 },
-        /* 26*/ { -1, "0000", 0, 1, 27 }, /* Defaults to channel 5 due to length */
-        /* 27*/ { -1, "3493", 0, 1, 27 },
-        /* 28*/ { 5, "3493", 0, 1, 27 },
-        /* 29*/ { 5, "3494", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 30*/ { -1, "3494", 0, 1, 31 }, /* Channel 6 */
-        /* 31*/ { 6, "00000", 0, 1, 31 },
-        /* 32*/ { -1, "00000", 0, 1, 31 }, /* Defaults to channel 5 due to length */
-        /* 33*/ { -1, "44072", 0, 1, 31 },
-        /* 34*/ { 6, "44072", 0, 1, 31 },
-        /* 35*/ { 6, "44073", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 36*/ { -1, "44073", 0, 1, 35 }, /* Channel 7 */
-        /* 37*/ { 7, "000000", 0, 1, 35 },
-        /* 38*/ { -1, "000000", 0, 1, 35 }, /* Defaults to channel 7 due to length */
-        /* 39*/ { 7, "576688", 0, 1, 35 },
-        /* 40*/ { 7, "576689", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 41*/ { 7, "0576688", 0, 1, 35 },
-        /* 42*/ { -1, "1234567", 0, 1, 39 },
-        /* 43*/ { -1, "576689", 0, 1, 39 }, /* Channel 8 */
-        /* 44*/ { 8, "0000000", 0, 1, 39, },
-        /* 45*/ { -1, "0000000", 0, 1, 39, }, /* Defaults to channel 8 due to length */
-        /* 46*/ { 8, "1234567", 0, 1, 39, },
-        /* 47*/ { 8, "7742863", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 48*/ { 8, "01234567", ZINT_ERROR_TOO_LONG, -1, -1 },
-        /* 49*/ { 8, "00000000", ZINT_ERROR_TOO_LONG, -1, -1 },
-        /* 50*/ { 9, "7742863", ZINT_ERROR_INVALID_DATA, -1, -1 },
-        /* 51*/ { -1, "A", ZINT_ERROR_INVALID_DATA, -1, -1 },
+    static const struct item data[] = {
+        /*  0*/ { -1, "0", 0, 1, 19, "" }, /* < 3 ignored */
+        /*  1*/ { 0, "0", 0, 1, 19, "" },
+        /*  2*/ { 1, "0", 0, 1, 19, "" },
+        /*  3*/ { 2, "0", 0, 1, 19, "" },
+        /*  4*/ { 9, "0", 0, 1, 19, "" }, /* > 8 ignored */
+        /*  5*/ { -1, "00", 0, 1, 19, "" },
+        /*  6*/ { 3, "00", 0, 1, 19, "" },
+        /*  7*/ { -1, "26", 0, 1, 19, "" },
+        /*  8*/ { 3, "26", 0, 1, 19, "" },
+        /*  9*/ { 3, "27", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"27\" out of range (0 to 26 for 3 channels)" },
+        /* 10*/ { 3, "027", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"27\" out of range (0 to 26 for 3 channels)" },
+        /* 11*/ { 3, "000", 0, 1, 19, "" },
+        /* 12*/ { 3, "001", 0, 1, 19, "" },
+        /* 13*/ { 3, "026", 0, 1, 19, "" },
+        /* 14*/ { -1, "27", 0, 1, 23, "" }, /* Channel 4 */
+        /* 15*/ { -1, "026", 0, 1, 23, "" }, /* Defaults to channel 4 due to length */
+        /* 16*/ { 3, "0026", 0, 1, 19, "" },
+        /* 17*/ { 3, "1234", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"1234\" out of range (0 to 26 for 3 channels)" },
+        /* 18*/ { 4, "000", 0, 1, 23, "" },
+        /* 19*/ { -1, "000", 0, 1, 23, "" }, /* Defaults to channel 4 due to length */
+        /* 20*/ { 4, "026", 0, 1, 23, "" },
+        /* 21*/ { 4, "0000026", 0, 1, 23, "" },
+        /* 22*/ { 4, "0000", 0, 1, 23, "" },
+        /* 23*/ { 4, "292", 0, 1, 23, "" },
+        /* 24*/ { 4, "293", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"293\" out of range (0 to 292 for 4 channels)" },
+        /* 25*/ { 4, "000293", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"293\" out of range (0 to 292 for 4 channels)" },
+        /* 26*/ { -1, "293", 0, 1, 27, "" }, /* Channel 5 */
+        /* 27*/ { 5, "0000", 0, 1, 27, "" },
+        /* 28*/ { -1, "0000", 0, 1, 27, "" }, /* Defaults to channel 5 due to length */
+        /* 29*/ { -1, "3493", 0, 1, 27, "" },
+        /* 30*/ { 5, "3493", 0, 1, 27, "" },
+        /* 31*/ { 5, "3494", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"3494\" out of range (0 to 3493 for 5 channels)" },
+        /* 32*/ { -1, "3494", 0, 1, 31, "" }, /* Channel 6 */
+        /* 33*/ { 6, "00000", 0, 1, 31, "" },
+        /* 34*/ { -1, "00000", 0, 1, 31, "" }, /* Defaults to channel 5 due to length */
+        /* 35*/ { -1, "44072", 0, 1, 31, "" },
+        /* 36*/ { 6, "44072", 0, 1, 31, "" },
+        /* 37*/ { 6, "44073", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"44073\" out of range (0 to 44072 for 6 channels)" },
+        /* 38*/ { -1, "44073", 0, 1, 35, "" }, /* Channel 7 */
+        /* 39*/ { 7, "000000", 0, 1, 35, "" },
+        /* 40*/ { -1, "000000", 0, 1, 35, "" }, /* Defaults to channel 7 due to length */
+        /* 41*/ { 7, "576688", 0, 1, 35, "" },
+        /* 42*/ { 7, "576689", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 335: Input value \"576689\" out of range (0 to 576688 for 7 channels)" },
+        /* 43*/ { 7, "0576688", 0, 1, 35, "" },
+        /* 44*/ { -1, "1234567", 0, 1, 39, "" },
+        /* 45*/ { -1, "576689", 0, 1, 39, "" }, /* Channel 8 */
+        /* 46*/ { 8, "0000000", 0, 1, 39, "" },
+        /* 47*/ { -1, "0000000", 0, 1, 39, "" }, /* Defaults to channel 8 due to length */
+        /* 48*/ { 8, "1234567", 0, 1, 39, "" },
+        /* 49*/ { 8, "7742863", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 318: Input value \"7742863\" out of range (0 to 7742862)" },
+        /* 50*/ { 8, "01234567", ZINT_ERROR_TOO_LONG, -1, -1, "Error 333: Input length 8 too long (maximum 7)" },
+        /* 51*/ { 8, "00000000", ZINT_ERROR_TOO_LONG, -1, -1, "Error 333: Input length 8 too long (maximum 7)" },
+        /* 52*/ { 9, "7742863", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 318: Input value \"7742863\" out of range (0 to 7742862)" },
+        /* 53*/ { -1, "A", ZINT_ERROR_INVALID_DATA, -1, -1, "Error 334: Invalid character at position 1 in input (digits only)" },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i, length, ret;
-    struct zint_symbol *symbol;
+    struct zint_symbol *symbol = NULL;
 
-    testStart("test_input");
+    testStartSymbol("test_input", &symbol);
 
     for (i = 0; i < data_size; i++) {
 
@@ -171,8 +193,10 @@ static void test_input(const testCtx *const p_ctx) {
 
         length = testUtilSetSymbol(symbol, BARCODE_CHANNEL, -1 /*input_mode*/, -1 /*eci*/, -1 /*option_1*/, data[i].option_2, -1, -1 /*output_options*/, data[i].data, -1, debug);
 
-        ret = ZBarcode_Encode(symbol, (unsigned char *) data[i].data, length);
+        ret = ZBarcode_Encode(symbol, TCU(data[i].data), length);
         assert_equal(ret, data[i].ret, "i:%d ZBarcode_Encode ret %d != %d\n", i, ret, data[i].ret);
+        assert_equal(symbol->errtxt[0] == '\0', ret == 0, "i:%d symbol->errtxt not %s (%s)\n", i, ret ? "set" : "empty", symbol->errtxt);
+        assert_zero(strcmp(symbol->errtxt, data[i].expected_errtxt), "i:%d strcmp(%s, %s) != 0\n", i, symbol->errtxt, data[i].expected_errtxt);
 
         if (ret < ZINT_ERROR) {
             assert_equal(symbol->rows, data[i].expected_rows, "i:%d symbol->rows %d != %d (%s)\n", i, symbol->rows, data[i].expected_rows, data[i].data);
@@ -192,15 +216,15 @@ static void test_encode(const testCtx *const p_ctx) {
 
     struct item {
         int option_2;
-        char *data;
+        const char *data;
         int ret;
 
         int expected_rows;
         int expected_width;
-        char *comment;
-        char *expected;
+        const char *comment;
+        const char *expected;
     };
-    struct item data[] = {
+    static const struct item data[] = {
         /*  0*/ { -1, "1234", 0, 1, 27, "ANSI/AIM BC12-1998 Figure 1",
                     "101010101001010010011110011"
                 },
@@ -412,15 +436,15 @@ static void test_encode(const testCtx *const p_ctx) {
                     "101010101000001110010101101100101110011"
                 },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i, length, ret;
-    struct zint_symbol *symbol;
+    struct zint_symbol *symbol = NULL;
 
     char escaped[1024];
     char bwipp_buf[8192];
     char bwipp_msg[1024];
 
-    testStart("test_encode");
+    testStartSymbol("test_encode", &symbol);
 
     for (i = 0; i < data_size; i++) {
 
@@ -431,7 +455,7 @@ static void test_encode(const testCtx *const p_ctx) {
 
         length = testUtilSetSymbol(symbol, BARCODE_CHANNEL, -1 /*input_mode*/, -1 /*eci*/, -1 /*option_1*/, data[i].option_2, -1, -1 /*output_options*/, data[i].data, -1, debug);
 
-        ret = ZBarcode_Encode(symbol, (unsigned char *) data[i].data, length);
+        ret = ZBarcode_Encode(symbol, TCU(data[i].data), length);
         assert_equal(ret, data[i].ret, "i:%d ZBarcode_Encode ret %d != %d (%s)\n", i, ret, data[i].ret, symbol->errtxt);
 
         if (p_ctx->generate) {
@@ -471,10 +495,10 @@ static void test_encode(const testCtx *const p_ctx) {
 static void test_generate(const testCtx *const p_ctx) {
 
     struct item {
-        char *data;
+        const char *data;
     };
-    struct item data[] = { { "576688" }, { "7742862" } };
-    int data_size = ARRAY_SIZE(data);
+    static const struct item data[] = { { "576688" }, { "7742862" } };
+    const int data_size = ARRAY_SIZE(data);
     int i, length, ret;
     struct zint_symbol *symbol;
 
@@ -488,7 +512,7 @@ static void test_generate(const testCtx *const p_ctx) {
 
         length = testUtilSetSymbol(symbol, BARCODE_CHANNEL, -1 /*input_mode*/, -1 /*eci*/, -1 /*option_1*/, -1, -1, -1 /*output_options*/, data[i].data, -1, 0);
 
-        ret = ZBarcode_Encode(symbol, (unsigned char *) data[i].data, length);
+        ret = ZBarcode_Encode(symbol, TCU(data[i].data), length);
         assert_zero(ret, "i:%d ZBarcode_Encode ret %d != 0 (%s)\n", i, ret, symbol->errtxt);
 
         ZBarcode_Delete(symbol);
