@@ -1,6 +1,6 @@
 /*
     libzint - the open source barcode library
-    Copyright (C) 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2021-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -27,18 +27,18 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 #include "testcommon.h"
 #include "../iso3166.h"
 
-static void test_numeric(int index) {
+static void test_numeric(const testCtx *const p_ctx) {
 
     struct item {
         int data;
         int ret;
     };
-    // s/\/\*[ 0-9]*\*\//\=printf("\/*%3d*\/", line(".") - line("'<"))
+    /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
     struct item data[] = {
         /*  0*/ { -1, 0 },
         /*  1*/ { 0, 0 },
@@ -381,7 +381,7 @@ static void test_numeric(int index) {
 
     for (i = 0; i < data_size; i++) {
 
-        if (index != -1 && i != index) continue;
+        if (testContinue(p_ctx, i)) continue;
 
         ret = iso3166_numeric(data[i].data);
         assert_equal(ret, data[i].ret, "i:%d ret %d != %d\n", i, ret, data[i].ret);
@@ -436,13 +436,15 @@ static int bc_iso3166_numeric(int cc) {
     return 0;
 }
 
-static void test_numeric_bc(void) {
+static void test_numeric_bc(const testCtx *const p_ctx) {
 
     int i, ret, bc_ret;
 
     testStart("test_numeric_bc");
 
     for (i = 0; i < 1001; i++) {
+        if (testContinue(p_ctx, i)) continue;
+
         ret = iso3166_numeric(i);
         bc_ret = bc_iso3166_numeric(i);
         assert_equal(ret, bc_ret, "i:%d ret %d != bc_ret %d\n", i, ret, bc_ret);
@@ -451,13 +453,13 @@ static void test_numeric_bc(void) {
     testFinish();
 }
 
-static void test_alpha2(int index) {
+static void test_alpha2(const testCtx *const p_ctx) {
 
     struct item {
         const char *data;
         int ret;
     };
-    // s/\/\*[ 0-9]*\*\//\=printf("\/*%3d*\/", line(".") - line("'<"))
+    /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
     struct item data[] = {
         /*  0*/ { "", 0 },
         /*  1*/ { "A", 0 },
@@ -681,7 +683,7 @@ static void test_alpha2(int index) {
 
     for (i = 0; i < data_size; i++) {
 
-        if (index != -1 && i != index) continue;
+        if (testContinue(p_ctx, i)) continue;
 
         ret = iso3166_alpha2(data[i].data);
         assert_equal(ret, data[i].ret, "i:%d ret %d != %d\n", i, ret, data[i].ret);
@@ -737,11 +739,13 @@ static int bc_iso3166_alpha2(const char *cc) {
     return 0;
 }
 
-static void test_alpha2_bc(void) {
+static void test_alpha2_bc(const testCtx *const p_ctx) {
 
     int i, ret, bc_ret;
     char data[2];
 
+    (void)p_ctx;
+
     testStart("test_alpha2_bc");
 
     for (i = 0; i < 128; i++) {
@@ -760,11 +764,11 @@ static void test_alpha2_bc(void) {
 
 int main(int argc, char *argv[]) {
 
-    testFunction funcs[] = { /* name, func, has_index, has_generate, has_debug */
-        { "test_numeric", test_numeric, 1, 0, 0 },
-        { "test_alpha2", test_alpha2, 1, 0, 0 },
-        { "test_numeric_bc", test_numeric_bc, 0, 0, 0 },
-        { "test_alpha2_bc", test_alpha2_bc, 0, 0, 0 },
+    testFunction funcs[] = { /* name, func */
+        { "test_numeric", test_numeric },
+        { "test_alpha2", test_alpha2 },
+        { "test_numeric_bc", test_numeric_bc },
+        { "test_alpha2_bc", test_alpha2_bc },
     };
 
     testRun(argc, argv, funcs, ARRAY_SIZE(funcs));
@@ -773,3 +777,5 @@ int main(int argc, char *argv[]) {
 
     return 0;
 }
+
+/* vim: set ts=4 sw=4 et : */
