@@ -1,7 +1,7 @@
 /* zint_tcl.c TCL binding for zint */
 /*
     zint - the open source tcl binding to the zint barcode library
-    Copyright (C) 2014-2022 Harald Oehlmann <oehhar@users.sourceforge.net>
+    Copyright (C) 2014-2025 Harald Oehlmann <oehhar@users.sourceforge.net>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -164,6 +164,24 @@
     *** Potential incompatibility ***
 2022-12-09 GL
 - Added UPU_S10
+2023-01-15 GL
+- Added -esc and -extraesc options
+2023-02-10 GL
+- Added -textgap option
+2023-08-11 GL
+- Added -guardwhitespace option
+2023-10-30 GL
+- Added -dmiso144 option
+2024-12-09 HaO
+- TCL 9 compatibility
+- support TCL buildinfo
+- remove the zint command on dll unload
+2024-12-23 GL
+- Added DXFILMEDGE
+2025-01-29 GL
+- MSVC: suppress warning 4996 (_CRT_SECURE_NO_WARNINGS)
+2025-02-15 GL
+- strcpy() -> memcpy(); sizeof(primary); tabs -> spaces
 */
 
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
@@ -179,6 +197,11 @@
 #ifdef ERROR_INVALID_DATA
 #undef ERROR_INVALID_DATA
 #endif
+
+#if defined(_MSC_VER) && _MSC_VER > 1200 /* VC6 */
+#pragma warning(disable: 4996) /* function or variable may be unsafe */
+#endif
+
 #endif
 
 #include <zint.h>
@@ -186,14 +209,20 @@
 #include "../backend/zintconfig.h"
 #include <string.h>
 
-#if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
-#define USE_TCL_STUBS
-#define USE_TK_STUBS
-#endif
-
 #include <tcl.h>
 #include <tk.h>
 
+/* TCL 9 compatibility for TCL 8.6 compile */
+#ifndef TCL_SIZE_MAX
+#ifndef Tcl_Size
+typedef int Tcl_Size;
+#endif
+# define Tcl_GetSizeIntFromObj Tcl_GetIntFromObj
+# define Tcl_NewSizeIntObj Tcl_NewIntObj
+# define TCL_SIZE_MAX      INT_MAX
+# define TCL_SIZE_MODIFIER ""
+#endif
+
 #undef EXPORT
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
 #define EXPORT __declspec(dllexport)
@@ -216,13 +245,13 @@
 
 /*----------------------------------------------------------------------------*/
 /* >>>> External Prototypes (exports) */
-EXPORT int Zint_Init (Tcl_Interp *interp);
-EXPORT int Zint_Unload (Tcl_Interp *Interp, int Flags);
+DLLEXPORT int Zint_Init (Tcl_Interp *interp);
+DLLEXPORT int Zint_Unload (Tcl_Interp *Interp, int Flags);
 /*----------------------------------------------------------------------------*/
 /* >>>> local prototypes */
 static void InterpCleanupProc(ClientData clientData, Tcl_Interp *interp);
 static int CheckForTk(Tcl_Interp *interp, int *tkFlagPtr);
-static int Zint(ClientData unused, Tcl_Interp *interp, int objc,
+static int ZintCmd(ClientData unused, Tcl_Interp *interp, int objc,
     Tcl_Obj *CONST objv[]);
 static int Encode(Tcl_Interp *interp, int objc,
     Tcl_Obj *CONST objv[]);
@@ -312,6 +341,7 @@ static const char *s_code_list[] = {
     "MailMark-2D",
     "UPU-S10",
     "MailMark-4S",
+    "DXFilmEdge",
     "AztecRunes",
     "Code32",
     "EAN-CC",
@@ -414,6 +444,7 @@ static const int s_code_number[] = {
     BARCODE_MAILMARK_2D,
     BARCODE_UPU_S10,
     BARCODE_MAILMARK_4S,
+    BARCODE_DXFILMEDGE,
     BARCODE_AZRUNE,
     BARCODE_CODE32,
     BARCODE_EANX_CC,
@@ -500,18 +531,21 @@ static const char help_message[] = "zint tcl(stub,obj) dll\n"
     "   -bindtop bool: bar above the code, size set by -border\n"
     "   -bold bool: use bold text\n"
     "   -border integer: width of a border around the symbol. Use with -bind/-box/-bindtop 1\n"
-    "   -box bool: box around bar code, size set be -border\n"
+    "   -box bool: box around bar code, size set by -border\n"
     /* cli option --cmyk not supported as no corresponding output */
     "   -cols integer: Codablock F, DotCode, PDF417: number of columns\n"
     "   -compliantheight bool: warn if height not compliant, and use standard default\n"
     /* cli option --data is standard parameter */
+    "   -dmiso144 bool: Use ISO format for 144x144 Data Matrix symbols\n"
     "   -dmre bool: Allow Data Matrix Rectangular Extended\n"
     "   -dotsize number: radius ratio of dots from 0.01 to 1.0\n" 
     "   -dotty bool: use dots instead of boxes for matrix codes\n"
     /* cli option --dump not supported */
     /* cli option --ecinos not supported */
     "   -eci choice: ECI to use\n"
-    /* cli option --esc not supported */
+    /* cli option --embedfont not supported (vector output only) */
+    "   -esc bool: Process escape sequences in input data\n"
+    "   -extraesc bool: Process symbology-specific escape sequences (Code 128 only)\n"
     "   -fast bool: use fast encodation (Data Matrix)\n"
     "   -fg color: set foreground color as 6 or 8 hex rrggbbaa\n"
     /* replaces cli options --binary and --gs1 */
@@ -522,6 +556,7 @@ static const char help_message[] = "zint tcl(stub,obj) dll\n"
     "   -gs1parens bool: for gs1, AIs enclosed in parentheses instead of square brackets\n"
     "   -gssep bool: for gs1, use gs as separator instead fnc1 (Datamatrix only)\n"
     "   -guarddescent double: Height of guard bar descent in modules (EAN/UPC only)\n"
+    "   -guardwhitespace bool: add quiet zone indicators (EAN/UPC only)\n"
     "   -height double: Symbol height in modules\n"
     "   -heightperrow bool: treat height as per-row\n"
     /* cli option --input not supported */
@@ -548,6 +583,7 @@ static const char help_message[] = "zint tcl(stub,obj) dll\n"
     "   -smalltext bool: tiny interpretation line font\n"
     "   -square bool: force Data Matrix symbols to be square\n"
     "   -structapp {index count ?id?}: set Structured Append info\n"
+    "   -textgap double: Gap between barcode and text\n"
     /* cli option --types not supported */
     "   -vers integer: Symbology option\n"
     /* cli option --version not supported */
@@ -574,19 +610,74 @@ EXPORT BOOL WINAPI DllEntryPoint (HINSTANCE hInstance,
 #endif
 /*----------------------------------------------------------------------------*/
 /* Initialisation Procedures */
-EXPORT int Zint_Init (Tcl_Interp *interp)
+DLLEXPORT int Zint_Init (Tcl_Interp *interp)
 {
-    int * tkFlagPtr;
+    int *tkFlagPtr;
+    Tcl_CmdInfo info;
     /*------------------------------------------------------------------------*/
-#ifdef USE_TCL_STUBS
-    if (Tcl_InitStubs(interp, "8.5", 0) == NULL)
-#else
-    if (Tcl_PkgRequire(interp, "Tcl", "8.5", 0) == NULL)
-#endif
-    {
+    /* If TCL_STUB is not defined, the following only does a version check    */
+    if (Tcl_InitStubs(interp, "8.5-", 0) == NULL) {
         return TCL_ERROR;
     }
     /*------------------------------------------------------------------------*/
+    /* Add build info                                                         */
+    if (Tcl_GetCommandInfo(interp, "::tcl::build-info", &info)) {
+    Tcl_CreateObjCommand(interp, "::zint::build-info",
+        info.objProc, (void *)(
+            PACKAGE_VERSION "+" STRINGIFY(SAMPLE_VERSION_UUID)
+#if defined(__clang__) && defined(__clang_major__)
+                ".clang-" STRINGIFY(__clang_major__)
+#if __clang_minor__ < 10
+                "0"
+#endif
+                STRINGIFY(__clang_minor__)
+#endif
+#if defined(__cplusplus) && !defined(__OBJC__)
+                ".cplusplus"
+#endif
+#ifndef NDEBUG
+                ".debug"
+#endif
+#if !defined(__clang__) && !defined(__INTEL_COMPILER) && defined(__GNUC__)
+                ".gcc-" STRINGIFY(__GNUC__)
+#if __GNUC_MINOR__ < 10
+                "0"
+#endif
+                STRINGIFY(__GNUC_MINOR__)
+#endif
+#ifdef __INTEL_COMPILER
+                ".icc-" STRINGIFY(__INTEL_COMPILER)
+#endif
+#ifdef TCL_MEM_DEBUG
+                ".memdebug"
+#endif
+#if defined(_MSC_VER)
+                ".msvc-" STRINGIFY(_MSC_VER)
+#endif
+#ifdef USE_NMAKE
+                ".nmake"
+#endif
+#ifndef TCL_CFG_OPTIMIZED
+                ".no-optimize"
+#endif
+#ifdef __OBJC__
+                ".objective-c"
+#if defined(__cplusplus)
+                "plusplus"
+#endif
+#endif
+#ifdef TCL_CFG_PROFILED
+                ".profile"
+#endif
+#ifdef PURIFY
+                ".purify"
+#endif
+#ifdef STATIC_BUILD
+                ".static"
+#endif
+        ), NULL);
+    }
+    /*------------------------------------------------------------------------*/
     /* This procedure is called once per thread and any thread local data     */
     /* should be allocated and initialized here (and not in static variables) */
     
@@ -595,7 +686,9 @@ EXPORT int Zint_Init (Tcl_Interp *interp)
     *tkFlagPtr = 0;
     Tcl_CallWhenDeleted(interp, InterpCleanupProc, (ClientData)tkFlagPtr);
     /*------------------------------------------------------------------------*/
-    Tcl_CreateObjCommand(interp, "zint", Zint, (ClientData)tkFlagPtr,
+    /* FIXME: to unload even on command rename, capture the token, put it in  */
+    /* the client data and use it to delete the command.                      */
+    Tcl_CreateObjCommand(interp, "zint", ZintCmd, (ClientData)tkFlagPtr,
             (Tcl_CmdDeleteProc *)NULL);
     Tcl_PkgProvide (interp, "zint", version_string);
     /*------------------------------------------------------------------------*/
@@ -612,8 +705,11 @@ static void InterpCleanupProc(ClientData clientData, Tcl_Interp *interp)
 /*----------------------------------------------------------------------------*/
 /* >>>> Unload Procedures */
 /*----------------------------------------------------------------------------*/
-EXPORT int Zint_Unload (Tcl_Interp *Interp, int Flags)
+DLLEXPORT int Zint_Unload (Tcl_Interp *Interp, int Flags)
 {
+    /* Remove created commands */
+    Tcl_DeleteCommand(Interp, "::zint::build-info");
+    Tcl_DeleteCommand(Interp, "zint");
     // Allow unload
     return TCL_OK;
 }
@@ -621,7 +717,7 @@ EXPORT int Zint_Unload (Tcl_Interp *Interp, int Flags)
 /* >>>>> Called routine */
 /*----------------------------------------------------------------------------*/
 /* Decode tcl commands */
-static int Zint(ClientData tkFlagPtr, Tcl_Interp *interp, int objc,
+static int ZintCmd(ClientData tkFlagPtr, Tcl_Interp *interp, int objc,
     Tcl_Obj *CONST objv[])
 {
     /* Option list and indexes */
@@ -706,13 +802,13 @@ static int CheckForTk(Tcl_Interp *interp, int *tkFlagPtr)
         return TCL_OK;
     }
     if (*tkFlagPtr == 0) {
-        if ( ! Tcl_PkgPresent(interp, "Tk", "8.5", 0) ) {
+        if ( ! Tcl_PkgPresent(interp, "Tk", "8.5-", 0) ) {
             Tcl_SetResult(interp, "package Tk not loaded", TCL_STATIC);
             return TCL_ERROR;
         }
     }
 #ifdef USE_TK_STUBS
-    if (*tkFlagPtr < 0 || Tk_InitStubs(interp, "8.5", 0) == NULL) {
+    if (*tkFlagPtr < 0 || Tk_InitStubs(interp, "8.5-", 0) == NULL) {
         *tkFlagPtr = -1;
         Tcl_SetResult(interp, "error initializing Tk", TCL_STATIC);
         return TCL_ERROR;
@@ -730,7 +826,7 @@ static int Encode(Tcl_Interp *interp, int objc,
     struct zint_symbol *my_symbol;
     Tcl_DString dsInput;
     char *pStr = NULL;
-    int lStr;
+    Tcl_Size lStr;
     Tcl_Encoding hZINTEncoding;
     int rotate_angle=0;
     int fError = 0;
@@ -750,8 +846,8 @@ static int Encode(Tcl_Interp *interp, int objc,
     int seg_count = 0;
     int seg_no;
     Tcl_Obj *pSegDataObjs[10] = {0};
-    Tcl_DString segInputs[10];
-    struct zint_seg segs[10];
+    Tcl_DString segInputs[10] = {{0}};
+    struct zint_seg segs[10] = {{0}};
     double xdim = 0.0;
     double resolution = 0.0;
     /*------------------------------------------------------------------------*/
@@ -780,27 +876,27 @@ static int Encode(Tcl_Interp *interp, int objc,
         /* Option list and indexes */
         static const char *optionList[] = {
             "-addongap", "-barcode", "-bg", "-bind", "-bindtop", "-bold", "-border", "-box",
-            "-cols", "-compliantheight", "-dmre", "-dotsize", "-dotty",
-            "-eci", "-fast", "-fg", "-format", "-fullmultibyte",
-            "-gs1nocheck", "-gs1parens", "-gssep", "-guarddescent",
+            "-cols", "-compliantheight", "-dmiso144", "-dmre", "-dotsize", "-dotty",
+            "-eci", "-esc", "-extraesc", "-fast", "-fg", "-format", "-fullmultibyte",
+            "-gs1nocheck", "-gs1parens", "-gssep", "-guarddescent", "-guardwhitespace",
             "-height", "-heightperrow", "-init", "-mask", "-mode",
             "-nobackground", "-noquietzones", "-notext", "-primary", "-quietzones",
             "-reverse", "-rotate", "-rows", "-scale", "-scalexdimdp", "-scmvv", "-secure",
             "-seg1", "-seg2", "-seg3", "-seg4", "-seg5", "-seg6", "-seg7", "-seg8", "-seg9",
             "-separator", "-smalltext", "-square", "-structapp",
-            "-to", "-vers", "-vwhitesp", "-werror", "-whitesp",
+            "-textgap", "-to", "-vers", "-vwhitesp", "-werror", "-whitesp",
             NULL};
         enum iOption {
             iAddonGap, iBarcode, iBG, iBind, iBindTop, iBold, iBorder, iBox,
-            iCols, iCompliantHeight, iDMRE, iDotSize, iDotty,
-            iECI, iFast, iFG, iFormat, iFullMultiByte,
-            iGS1NoCheck, iGS1Parens, iGSSep, iGuardDescent,
+            iCols, iCompliantHeight, iDMISO144, iDMRE, iDotSize, iDotty,
+            iECI, iEsc, iExtraEsc, iFast, iFG, iFormat, iFullMultiByte,
+            iGS1NoCheck, iGS1Parens, iGSSep, iGuardDescent, iGuardWhitespace,
             iHeight, iHeightPerRow, iInit, iMask, iMode,
             iNoBackground, iNoQuietZones, iNoText, iPrimary, iQuietZones,
             iReverse, iRotate, iRows, iScale, iScaleXdimDp, iSCMvv, iSecure,
             iSeg1, iSeg2, iSeg3, iSeg4, iSeg5, iSeg6, iSeg7, iSeg8, iSeg9,
             iSeparator, iSmallText, iSquare, iStructApp,
-            iTo, iVers, iVWhiteSp, iWError, iWhiteSp
+            iTextGap, iTo, iVers, iVWhiteSp, iWError, iWhiteSp
             };
         int optionIndex;
         int intValue;
@@ -822,12 +918,16 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iBold:
         case iBox:
         case iCompliantHeight:
+        case iDMISO144:
         case iDMRE:
         case iDotty:
+        case iEsc:
+        case iExtraEsc:
         case iFast:
         case iGS1NoCheck:
         case iGS1Parens:
         case iGSSep:
+        case iGuardWhitespace:
         case iHeightPerRow:
         case iInit:
         case iNoBackground:
@@ -860,6 +960,7 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iGuardDescent:
         case iDotSize:
         case iScale:
+        case iTextGap:
             /* >> Float */
             if (TCL_OK != Tcl_GetDoubleFromObj(interp, objv[optionPos+1],
                 &doubleValue))
@@ -888,12 +989,11 @@ static int Encode(Tcl_Interp *interp, int objc,
             }
             break;
         case iPrimary:
-            /* > Primary String up to 90 characters */
-            /* > Output filename up to 250 characters */
+            /* > Primary String up to 127 characters */
             Tcl_DStringInit(& dString);
             pStr = Tcl_GetStringFromObj(objv[optionPos+1], &lStr);
             Tcl_UtfToExternalDString( hZINTEncoding, pStr, lStr, &dString);
-            if (Tcl_DStringLength(&dString) > (optionIndex==iPrimary?90:250)) {
+            if (Tcl_DStringLength(&dString) >= (int) sizeof(my_symbol->primary)) {
                 Tcl_DStringFree(&dString);
                 Tcl_SetObjResult(interp,Tcl_NewStringObj("String too long", -1));
                 fError = 1;
@@ -997,6 +1097,20 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->output_options &= ~BARCODE_DOTTY_MODE;
             }
             break;
+        case iEsc:
+            if (intValue) {
+                my_symbol->input_mode |= ESCAPE_MODE;
+            } else {
+                my_symbol->input_mode &= ~ESCAPE_MODE;
+            }
+            break;
+        case iExtraEsc:
+            if (intValue) {
+                my_symbol->input_mode |= EXTRA_ESCAPE_MODE;
+            } else {
+                my_symbol->input_mode &= ~EXTRA_ESCAPE_MODE;
+            }
+            break;
         case iFast:
             if (intValue) {
                 my_symbol->input_mode |= FAST_MODE;
@@ -1038,6 +1152,13 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->eci = s_eci_number[ECIIndex];
             }
             break;
+        case iGuardWhitespace:
+            if (intValue) {
+                my_symbol->output_options |= EANUPC_GUARD_WHITESPACE;
+            } else {
+                my_symbol->output_options &= ~EANUPC_GUARD_WHITESPACE;
+            }
+            break;
         case iHeightPerRow:
             if (intValue) {
                 my_symbol->input_mode |= HEIGHTPERROW_MODE;
@@ -1061,8 +1182,8 @@ static int Encode(Tcl_Interp *interp, int objc,
             break;
         case iReverse:
             if (intValue) {
-                strcpy(my_symbol->fgcolour, "ffffff");
-                strcpy(my_symbol->bgcolour, "000000");
+                memcpy(my_symbol->fgcolour, "ffffff", 7); /* Include terminating NUL */
+                memcpy(my_symbol->bgcolour, "000000", 7);
             }
             break;
         case iWError:
@@ -1080,7 +1201,7 @@ static int Encode(Tcl_Interp *interp, int objc,
             break;
         case iNoBackground:
             if (intValue) {
-                strcpy(my_symbol->bgcolour, "ffffff00");
+                memcpy(my_symbol->bgcolour, "ffffff00", 9); /* Include terminating NUL */
             }
             break;
         case iNoQuietZones:
@@ -1103,12 +1224,16 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iSquare:
             /* DM_SQUARE overwrites DM_DMRE */
             if (intValue)
-                my_symbol->option_3 = DM_SQUARE;
+                my_symbol->option_3 = DM_SQUARE | (my_symbol->option_3 & ~0x7F);
             break;
         case iDMRE:
             /* DM_DMRE overwrites DM_SQUARE */
             if (intValue)
-                my_symbol->option_3 = DM_DMRE;
+                my_symbol->option_3 = DM_DMRE | (my_symbol->option_3 & ~0x7F);
+            break;
+        case iDMISO144:
+            if (intValue)
+                my_symbol->option_3 |= DM_ISO_144;
             break;
         case iScale:
             if (doubleValue < 0.01) {
@@ -1119,6 +1244,15 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->scale = (float)doubleValue;
             }
             break;
+        case iTextGap:
+            if (doubleValue < 0.0 || doubleValue > 5.0) {
+                Tcl_SetObjResult(interp,
+                    Tcl_NewStringObj("Text Gap out of range", -1));
+                fError = 1;
+            } else {
+                my_symbol->text_gap = (float)doubleValue;
+            }
+            break;
         case iScaleXdimDp:
             /* >> Decode the -scalexdimdp parameter as list of xdim ?resolution? */
             {
@@ -1239,7 +1373,8 @@ static int Encode(Tcl_Interp *interp, int objc,
             }
             break;
         case iPrimary:
-            strcpy(my_symbol->primary, Tcl_DStringValue( &dString ) );
+            /* Include terminating NUL */
+            memcpy(my_symbol->primary, Tcl_DStringValue(&dString), Tcl_DStringLength(&dString) + 1);
             Tcl_DStringFree(&dString);
             break;
         case iRotate:
@@ -1287,7 +1422,7 @@ static int Encode(Tcl_Interp *interp, int objc,
                 Tcl_Obj *poParam;
                 struct zint_structapp structapp = { 0, 0, "" };
                 char *pStructAppId = NULL;
-                int lStructAppId = 0;
+                Tcl_Size lStructAppId = 0;
                 if (TCL_OK != Tcl_ListObjLength(interp,
                     objv[optionPos+1], &lStr))
                 {
@@ -1459,7 +1594,7 @@ static int Encode(Tcl_Interp *interp, int objc,
         }
         if (seg_count) {
             segs[0].source = (unsigned char *) pStr;
-            segs[0].length = lStr;
+            segs[0].length = (int)lStr;
             segs[0].eci = my_symbol->eci;
             for (seg_no = 1; seg_no < seg_count; seg_no++) {
                 if (!pSegDataObjs[seg_no]) {
@@ -1471,14 +1606,16 @@ static int Encode(Tcl_Interp *interp, int objc,
             if (!fError) {
                 for (seg_no = 1; seg_no < seg_count; seg_no++) {
                     if ((my_symbol->input_mode & 0x07) == DATA_MODE) {
+                        Tcl_Size LengthTemp;
                         segs[seg_no].source = (unsigned char *) Tcl_GetByteArrayFromObj(pSegDataObjs[seg_no],
-                            &segs[seg_no].length);
+                            &LengthTemp);
+                        segs[seg_no].length = (int)LengthTemp;
                     } else {
                         pStr = Tcl_GetStringFromObj(pSegDataObjs[seg_no], &lStr);
                         Tcl_DStringInit(& segInputs[seg_no]);
                         Tcl_UtfToExternalDString( hZINTEncoding, pStr, lStr, &segInputs[seg_no]);
                         segs[seg_no].source = (unsigned char *) Tcl_DStringValue( &segInputs[seg_no] );
-                        segs[seg_no].length = Tcl_DStringLength( &segInputs[seg_no] );
+                        segs[seg_no].length = (int)Tcl_DStringLength( &segInputs[seg_no] );
                     }
                 }
             }
@@ -1496,7 +1633,7 @@ static int Encode(Tcl_Interp *interp, int objc,
                 segs, seg_count, rotate_angle);
         } else {
             ErrorNumber = ZBarcode_Encode_and_Buffer(my_symbol,
-                (unsigned char *) pStr, lStr, rotate_angle);
+                (unsigned char *) pStr, (int)lStr, rotate_angle);
         }
         /*--------------------------------------------------------------------*/
         /* >> Show a message */
@@ -1515,8 +1652,8 @@ static int Encode(Tcl_Interp *interp, int objc,
                 Tcl_NewStringObj("Unknown photo image", -1));
             fError = 1;
         } else {
-            Tk_PhotoImageBlock sImageBlock;
-            char * pImageRGBA = NULL;
+            Tk_PhotoImageBlock sImageBlock = {0};
+            char *pImageRGBA = NULL;
             if (my_symbol->alphamap == NULL) {
                 sImageBlock.pixelPtr = (unsigned char *) my_symbol->bitmap;
                 sImageBlock.width = my_symbol->bitmap_width;
